-- GETS

drop procedure if exists e3tmqtt.WSS_get_device_validate;
delimiter $$
create procedure e3tmqtt.WSS_get_device_validate ( idconnection_var INT, mac_var VARCHAR(45) )
begin
	-- CALL e3tmqtt.WSS_get_device_validate ( 127, '00-1E-C0-0F-A2-F8' )
	-- CALL e3tmqtt.WSS_get_device_validate ( 50, '21-21-21-21-21' )

	DECLARE result_var INT DEFAULT 0;
	DECLARE idplatform_var INT DEFAULT NULL;
	DECLARE idgroup_var INT DEFAULT NULL;
	DECLARE iddevice_var INT DEFAULT NULL;
	DECLARE RESTWS_location_var VARCHAR(200) DEFAULT NULL;

	SELECT 1, G.idplatform, D.idgroup, D.iddevice, P.RESTWS_location
	FROM devices D 
	INNER JOIN `groups` G ON G.idgroup=D.idgroup AND G.idplatform=D.idplatform
	INNER JOIN platforms P ON P.id=G.idplatform
	WHERE D.mac=mac_var 
	ORDER BY D.idplatform LIMIT 1
	INTO result_var, idplatform_var, idgroup_var, iddevice_var, RESTWS_location_var;

	if(idconnection_var=0)
	then
		set idconnection_var=(select idconnection from messages_log where message like CONCAT('%', mac_var, '%') AND idplatform is null and idgroup is null and iddevice is null order by id desc limit 1);
	end if;

	CALL e3tmqtt.WSS_post_device_log ( idconnection_var, idplatform_var, idgroup_var, iddevice_var, mac_var, result_var );
	
	SELECT result_var result,idplatform_var idplatform, idgroup_var idgroup, iddevice_var iddevice, RESTWS_location_var RESTWS_location, install_restws_location, idconnection_var idconnection 
	FROM config;
end $$ 
delimiter ;


drop procedure if exists e3tmqtt.MQTT_get_device_validate;
delimiter $$
create procedure e3tmqtt.MQTT_get_device_validate ( mac_var VARCHAR(45) )
begin
	-- CALL e3tmqtt.MQTT_get_device_validate ( '00-1E-C0-0F-A2-F8' )

	DECLARE result_var INT DEFAULT 0;
	DECLARE idplatform_var INT DEFAULT NULL;
	DECLARE idgroup_var INT DEFAULT NULL;
	DECLARE iddevice_var INT DEFAULT NULL;
	DECLARE pass_var VARCHAR(200) DEFAULT NULL;

	if mac_var=(SELECT mqtt_server_name FROM config limit 1)
	then
		SET @pass=get_token(mac_var, mac_var);
		SET @result=1;

		CALL e3tmqtt.WSS_post_device_log ( @pass, 0, 0, 0, 0, mac_var, @result );
	else
		SELECT 1, G.idplatform, D.idgroup, D.iddevice, get_token( D.mac, D.mac ) pass
		FROM devices D
		INNER JOIN `groups` G ON G.idgroup=D.idgroup
		WHERE D.mac=mac_var	
		ORDER BY D.idplatform LIMIT 1
		INTO result_var, idplatform_var, idgroup_var, iddevice_var, pass_var;

		CALL e3tmqtt.WSS_post_device_log ( pass_var, 0, idplatform_var, idgroup_var, iddevice_var, mac_var, result_var );

	end if;

	if(@result=0)
	then
		SET pass_var=get_token(mac_var, mac_var);
	end if;

	SELECT 'PBKDF2$sha256$901$SPCW2NbWwYdk44fC$4acm8WxwC8l2ZuL3yBNUB7KpO12LxmKT' pw;
	-- SELECT @pass pw;
end $$ 
delimiter ;


drop procedure if exists e3tmqtt.MQTT_get_superuser_validate;
delimiter $$
create procedure e3tmqtt.MQTT_get_superuser_validate ( superuser_var VARCHAR(45) )
begin
	-- CALL e3tmqtt.MQTT_get_superuser_validate ( 'e3t.MQTT.Server' )
	-- CALL e3tmqtt.MQTT_get_superuser_validate ( 'AnonymouS' )

	SELECT COUNT(*) FROM config WHERE mqtt_server_name=superuser_var;
	
end $$ 
delimiter ;


drop procedure if exists e3tmqtt.MQTT_get_acls_topics;
delimiter $$
create procedure e3tmqtt.MQTT_get_acls_topics ( mac_var VARCHAR(45), rw_var INT )
begin
	-- CALL e3tmqtt.MQTT_get_acls_topics ( '00-1E-C0-0F-A2-F8' , 1 )
	-- CALL e3tmqtt.MQTT_get_acls_topics ( '00-1E-C0-0F-A2-F8' , 2 )

	if rw_var=1
	then
		select topic from (select '/Device/#' topic from devices where mac=mac_var and status=1 order by idplatform limit 1) device
		union
		select '/test' topic;
	else
		select topic from (select '/Device_Connections' topic from devices where mac=mac_var and status=1 order by idplatform limit 1) device_connections
		union
		select topic from (select '/LastWills' topic from devices where mac=mac_var and status=1 order by idplatform limit 1) lastwills
		union
		select '/Device_Installed' topic
		union
		select '/test' topic;
	end if;
end $$ 
delimiter ;





drop procedure if exists e3tmqtt.WSS_get_device_token_check;
delimiter $$
create procedure e3tmqtt.WSS_get_device_token_check ( token_var VARCHAR(50), date_mills_var BIGINT )
begin
	-- CALL e3tmqtt.WSS_get_device_token_check ( 'dedf23e3fbf505194e627813b325f4e9', '1463565774680' )

	DECLARE result_var INT DEFAULT 0;
	DECLARE idplatform_var INT DEFAULT NULL;
	DECLARE idgroup_var INT DEFAULT NULL;
	DECLARE iddevice_var INT DEFAULT NULL;
	DECLARE mac_var INT DEFAULT NULL;
	DECLARE RESTWS_location_var VARCHAR(200) DEFAULT NULL;

	SELECT 1 result, G.idplatform, D.idgroup, D.iddevice, D.mac, P.RESTWS_location
	FROM devices D 
	INNER JOIN `groups` G ON G.idgroup=D.idgroup AND G.idplatform=D.idplatform
	INNER JOIN platforms P ON P.id=G.idplatform
	WHERE get_token( D.mac, date_mills_var ) = token_var AND (D.token_connection_date is null OR D.token_connection_date<date_mills_var)
	ORDER BY D.idplatform LIMIT 1
	INTO result_var, idplatform_var, idgroup_var, iddevice_var, mac_var, RESTWS_location_var;


	CALL e3tmqtt.WSS_post_device_log ( token_var, date_mills_var, idplatform_var, idgroup_var, iddevice_var, mac_var, result_var );
	
	SELECT result_var result, idplatform_var idplatform, idgroup_var idgroup, iddevice_var iddevice, mac_var mac, RESTWS_location_var RESTWS_location, install_restws_location
	FROM config;
end $$ 
delimiter ;


drop procedure if exists e3tmqtt.WSS_get_device_token_check_feria;
delimiter $$
create procedure e3tmqtt.WSS_get_device_token_check_feria ( iddevice_var INT )
begin
	-- CALL e3tmqtt.WSS_get_device_token_check_feria ( 136 )

	DECLARE result_var INT DEFAULT 0;
	DECLARE idplatform_var INT DEFAULT NULL;
	DECLARE idgroup_var INT DEFAULT NULL;
	DECLARE mac_var INT DEFAULT NULL;
	DECLARE RESTWS_location_var VARCHAR(200) DEFAULT NULL;

	SELECT 1 result, G.idplatform, D.idgroup, D.iddevice, D.mac, P.RESTWS_location
	FROM devices D 
	INNER JOIN `groups` G ON G.idgroup=D.idgroup AND G.idplatform=D.idplatform
	INNER JOIN platforms P ON P.id=G.idplatform
	WHERE D.iddevice=iddevice_var
	ORDER BY D.idplatform LIMIT 1
 	INTO result_var, idplatform_var, idgroup_var, iddevice_var, mac_var, RESTWS_location_var;

	UPDATE devices SET status=1 WHERE iddevice=iddevice_var;

	CALL e3tmqtt.WSS_post_device_log ( iddevice_var, now(), idplatform_var, idgroup_var, iddevice_var, mac_var, result_var );
	
	SELECT result_var result, idplatform_var idplatform, idgroup_var idgroup, iddevice_var iddevice, mac_var mac, RESTWS_location_var RESTWS_location, install_restws_location, get_date_mills() date_mills
	FROM config;
end $$ 
delimiter ;


drop procedure if exists e3tmqtt.MQTT_get_device;
delimiter $$
create procedure e3tmqtt.MQTT_get_device ( idplatform_var INT, iddevice_var INT )
begin
	-- CALL e3tmqtt.MQTT_get_device ( 9, 109019396 )

	DECLARE result_var INT DEFAULT 0;
	DECLARE idgroup_var INT DEFAULT NULL;
    DECLARE idtype_var INT DEFAULT NULL;
	DECLARE mac_var VARCHAR(50) DEFAULT NULL;
	DECLARE RESTWS_location_var VARCHAR(200) DEFAULT NULL;

	SET @status_pre = NULL;

	SELECT 	1 result, G.idplatform, D.idgroup, D.iddevice, D.idtype, D.mac, 
			IFNULL(D.RESTWS_location, P.RESTWS_location) RESTWS_location,
			D.status
	FROM devices D 
	INNER JOIN `groups` G ON G.idgroup=D.idgroup AND G.idplatform=D.idplatform
	INNER JOIN platforms P ON P.id=G.idplatform
	WHERE D.iddevice=iddevice_var AND D.idplatform=idplatform_var
	ORDER BY D.idplatform LIMIT 1
 	INTO result_var, idplatform_var, idgroup_var, iddevice_var, idtype_var, mac_var, RESTWS_location_var, @status_pre;


	if(@status_pre!=1)
	then
		UPDATE devices SET status=1 WHERE iddevice=iddevice_var AND idplatform=idplatform_var;
		
		CALL e3tmqtt.WSS_post_device_log ( iddevice_var, now(), idplatform_var, idgroup_var, iddevice_var, mac_var, result_var );
	end if;

	if idplatform_var = 7
    then
		-- SIEMPRE ENTREGAMOS LOS MENSAJES DE CONFIGURACION
        SET @status_pre=-10;
    end if;
	
	SELECT 	result_var result, idplatform_var idplatform, idgroup_var idgroup, iddevice_var iddevice, idtype_var idtype, mac_var mac, RESTWS_location_var RESTWS_location, install_restws_location, get_date_mills() date_mills, firmware_sent_date_last, D.datalogger_fields, D.measures_fields, D.events_fields, 
			IF(config_device_date > config_device_date_update AND D.config_device_request_allways=0, config_device, '') config_device, 
			IF(config_params_date > config_params_date_update AND D.config_params_request_allways=0, config_params, '') config_params, 
			IF(config_output_date > config_output_date_update AND D.config_output_request_allways=0, config_output, '') config_output,
			D.connection_hwid, D.connection_device, D.firmware_sent_date_last, C.MQTT_debug_log, D.config_serial_response, D.secured,
			D.mqttsn, @status_pre status_pre -- , D.lwm2m
	FROM devices D
	INNER JOIN devices_types DT ON DT.id=D.idtype 
	INNER JOIN config C 
	WHERE D.iddevice=iddevice_var AND D.idplatform=idplatform_var
	LIMIT 1;
    
	-- SELECT LP.* FROM lwm2m_parser LP INNER JOIN devices D ON D.idtype=LP.iddevice_type WHERE D.iddevice=iddevice_var AND D.idplatform=idplatform_var;
end $$ 
delimiter ;



drop procedure if exists e3tmqtt.UDP_get_device;
delimiter $$
create procedure e3tmqtt.UDP_get_device ( imei_mac_var VARCHAR(50) )
begin
	-- CALL e3tmqtt.UDP_get_device ( 356345080030314 )

	DECLARE result_var INT DEFAULT 0;
	DECLARE idgroup_var INT DEFAULT NULL;
	DECLARE mac_var VARCHAR(50) DEFAULT NULL;
	DECLARE RESTWS_location_var VARCHAR(200) DEFAULT NULL;

	DECLARE idplatform_var INT;    DECLARE iddevice_var INT;
    SELECT idplatform, iddevice FROM devices WHERE mac=imei_mac_var
    INTO idplatform_var, iddevice_var;

	CALL e3tmqtt.MQTT_get_device ( idplatform_var, iddevice_var );

end $$ 
delimiter ;




drop procedure if exists e3tmqtt.MQTT_get_devices_online;
delimiter $$
create procedure e3tmqtt.MQTT_get_devices_online ( idplatform_var INT, iddevices_var INT )
begin
	-- CALL e3tmqtt.MQTT_get_devices_online ( 1, '55,8' )

	SELECT 1 result, D.idplatform, D.idgroup, iddevice, mac, P.RESTWS_location, P.APPRESTWS_location
	FROM devices D
	INNER JOIN devices_types DT ON DT.id=D.idtype
	INNER JOIN platforms P ON P.id=idplatform_var
	WHERE D.iddevice in (iddevices_var) AND D.idplatform=idplatform_var AND D.status=1;
end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.MQTT_get_mqtt_monitor;
$$
create procedure e3tmqtt.MQTT_get_mqtt_monitor ( idplatform_var INT, idgroup_var INT, iddevice_var INT, rows_var INT, date_start_var DATE, date_end_var DATE )
proc_label:begin
	-- CALL e3tmqtt.MQTT_get_mqtt_monitor ( 1, 1, 1, 0, '2018-07-01', '2018-07-06' )


	SET SQL_SAFE_UPDATES = 0;


	if(iddevice_var = 0)
	then
		set iddevice_var = null;
	end if;

	if(idgroup_var = 0)
	then
		set idgroup_var = null;
	end if;


	if idgroup_var is not null
	then
		DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups;
		CREATE TEMPORARY TABLE Temp_Table_groups (id int NOT NULL PRIMARY KEY AUTO_INCREMENT, idgroup int NOT NULL);

		CALL e3tmqtt.Hierarchical_group_subgroups ( idplatform_var, idgroup_var );
	end if;

	DROP TEMPORARY TABLE IF EXISTS temp_table_messages;

	if iddevice_var is null and idgroup_var is not null
	then
		SET @consulta= 'CREATE TEMPORARY TABLE temp_table_messages (INDEX date (date DESC))
						SELECT L.* 
								FROM messages_log L
								INNER JOIN devices D ON D.iddevice=L.iddevice AND D.idplatform=L.idplatform
								INNER JOIN Temp_Table_groups TTG ON TTG.idgroup=D.idgroup
								WHERE 1=1';
	elseif iddevice_var is not null
	then
		SET @consulta= CONCAT('CREATE TEMPORARY TABLE temp_table_messages (INDEX date (date DESC))
								SELECT * 
								FROM messages_log 
								WHERE idplatform=', idplatform_var,' AND iddevice=', iddevice_var);

	else
		SET @consulta= 'CREATE TEMPORARY TABLE temp_table_messages (INDEX date (date DESC))
								SELECT * 
								FROM messages_log 
								WHERE 1=1';
	end if;


	if idgroup_var is not null
	then
		SET @consulta2= CONCAT('INSERT temp_table_messages
					SELECT L.* 
							FROM messages_log L
							INNER JOIN Temp_Table_groups TTG ON TTG.idgroup=L.idgroup 
							WHERE L.iddevice=0 AND L.idplatform=', idplatform_var);
	end if;




	if(date_start_var!=0)
	then
		SET @consulta=CONCAT(@consulta, '
							AND date >= ''', date_start_var,'''');
		if idgroup_var is not null
		then
			SET @consulta2=CONCAT(@consulta2, '
							AND date >= ''', date_start_var,'''');
		end if;
	end if;

	if(date_end_var!=0)
	then
		SET @consulta=CONCAT(@consulta, '
							AND date <= DATE_ADD(''', date_end_var, ''', INTERVAL 1 DAY)');
		if idgroup_var is not null
		then
			SET @consulta2=CONCAT(@consulta2, '
							AND date <= DATE_ADD(''', date_end_var, ''', INTERVAL 1 DAY)');
		end if;
	end if;

	SET @consulta=CONCAT(@consulta, '
							ORDER BY date DESC, id DESC');

	if(rows_var!=0)
	then
		SET @consulta=CONCAT(@consulta, '
							LIMIT ', rows_var);
		if idgroup_var is not null
		then
			SET @consulta2=CONCAT(@consulta2, '
							LIMIT ', rows_var);
		end if;
	end if;


	-- preparamos el objete Statement a partir de nuestra variable
	PREPARE smpt FROM @consulta;
	-- ejecutamos el Statement
	EXECUTE smpt;
	-- liberamos la memoria
	DEALLOCATE PREPARE smpt;

	if idgroup_var is not null
	then
		-- preparamos el objete Statement a partir de nuestra variable
		PREPARE smpt2 FROM @consulta2;
		-- ejecutamos el Statement
		EXECUTE smpt2;
		-- liberamos la memoria
		DEALLOCATE PREPARE smpt2;
	end if;

	SET @consulta='SELECT * FROM temp_table_messages ORDER BY date DESC, id DESC';

	if(rows_var!=0)
	then
		SET @consulta=CONCAT(@consulta, '
							LIMIT ', rows_var);
	end if;

	-- preparamos el objete Statement a partir de nuestra variable
	PREPARE smpt FROM @consulta;
	-- ejecutamos el Statement
	EXECUTE smpt;
	-- liberamos la memoria
	DEALLOCATE PREPARE smpt;

	-- select @consulta2;

end $$



delimiter $$
drop procedure if exists e3tmqtt.WSS_get_device_check_mac;
$$
create procedure e3tmqtt.WSS_get_device_check_mac ( mac_var INT )
begin
	-- CALL e3tmqtt.WSS_get_device_check_mac ( '11-22-33-44-55' )

	DECLARE result_var INT DEFAULT 0;
	DECLARE idplatform_var INT DEFAULT NULL;
	DECLARE idgroup_var INT DEFAULT NULL;
	DECLARE iddevice_var INT DEFAULT NULL;
	DECLARE mac_var INT DEFAULT NULL;
	DECLARE RESTWS_location_var VARCHAR(200) DEFAULT NULL;

	SELECT 1 result, G.idplatform, D.idgroup, D.iddevice, D.mac, P.RESTWS_location
	FROM devices D 
	INNER JOIN `groups` G ON G.idgroup=D.idgroup AND G.idplatform=D.idplatform
	INNER JOIN platforms P ON P.id=G.idplatform
	WHERE D.mac=mac_var
	ORDER BY D.idplatform LIMIT 1
 	INTO result_var, idplatform_var, idgroup_var, iddevice_var, mac_var, RESTWS_location_var;

	UPDATE devices SET status=1 WHERE iddevice=iddevice_var;

	CALL e3tmqtt.WSS_post_device_log ( iddevice_var, now(), idplatform_var, idgroup_var, iddevice_var, mac_var, result_var );
	
	SELECT result_var result, idplatform_var idplatform, idgroup_var idgroup, iddevice_var iddevice, mac_var mac, RESTWS_location_var RESTWS_location, install_restws_location, get_date_mills() date_mills
	FROM config;
end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.WSS_get_device_token_check_lastwill;
$$
create procedure e3tmqtt.WSS_get_device_token_check_lastwill ( token_var VARCHAR(50), date_mills_var BIGINT )
begin
	-- CALL e3tmqtt.WSS_get_device_token_check_lastwill ( '52718d1308f8d106d1f65f3f0e579d30', '2016-05-18 09:19:20' )

	SET @result = 0;

	SELECT 1, G.idplatform, D.idgroup, D.iddevice, D.mac, P.RESTWS_location
	FROM devices D 
	INNER JOIN `groups` G ON G.idgroup=D.idgroup AND G.idplatform=D.idplatform
	INNER JOIN platforms P ON P.id=G.idplatform
	WHERE get_token( D.mac, date_mills_var ) = token_var AND (D.token_connection_date is null OR D.token_connection_date>date_mills_var)
	ORDER BY D.idplatform LIMIT 1
	INTO @result, @idplatform, @idgroup, @iddevice, @mac, @RESTWS_location;


	CALL e3tmqtt.WSS_post_device_log ( token_var, date_mills_var, @idplatform, @idgroup, @iddevice, @mac, @result );
	
	SELECT @result result, @idplatform idplatform, @idgroup idgroup, @iddevice iddevice, @mac mac, @RESTWS_location RESTWS_location, install_restws_location 
	FROM config;
end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.WSS_get_command_devices;
$$
create procedure e3tmqtt.WSS_get_command_devices ( idplatform_var INT, idgroup_var INT, iddevice_var INT, subgroups_var INT, iddevice_type_var INT, iddashboard_type_var INT, command_var TEXT )
begin
	-- CALL e3tmqtt.WSS_get_command_devices ( 1, 65, 0, 1, 0 )
	-- CALL e3tmqtt.WSS_get_command_devices ( 4, 2, 0, 1, 0 )
	-- CALL e3tmqtt.WSS_get_command_devices ( '1', '3', '0', '1', '0', '0', '{"command":"OUTPUT-STATUS","idgroup":"0","iddevice":"2","parameters":{"idstatus":"1","timer":"0","power":"100"}}' )

	SET SQL_SAFE_UPDATES = 0;

	DROP TEMPORARY TABLE IF EXISTS Temp_Table_devices;
	CREATE TEMPORARY TABLE Temp_Table_devices (iddevice int NOT NULL, token VARCHAR(200), token_date DATETIME, mac VARCHAR(50), status INT);

	if  (idplatform_var>0 and idgroup_var=0 and iddevice_var>0)
	then
		insert Temp_Table_devices
		select D.iddevice, D.token_connection, D.token_connection_date, D.mac, D.status 
		from devices D 
		inner join `groups` G on G.idgroup=D.idgroup
		inner join devices_types DT on DT.id=D.idtype
		where 	D.iddevice=iddevice_var and G.idplatform=idplatform_var and D.idplatform=idplatform_var 
				AND (0=iddevice_type_var OR D.idtype=iddevice_type_var)
				AND (0=iddashboard_type_var OR DT.iddashboard_type=iddashboard_type_var);

	elseif  (idplatform_var>0 and iddevice_var>0)
	then
		insert Temp_Table_devices
		select D.iddevice, D.token_connection, D.token_connection_date, D.mac, D.status
		from devices D 
		inner join `groups` G on G.idgroup=D.idgroup 
		inner join devices_types DT on DT.id=D.idtype
		where 	D.iddevice=iddevice_var and G.idplatform=idplatform_var and D.idplatform=idplatform_var
				AND (0=iddevice_type_var OR D.idtype=iddevice_type_var)
				AND (0=iddashboard_type_var OR DT.iddashboard_type=iddashboard_type_var);

	elseif (idplatform_var>0 and idgroup_var>0 and iddevice_var=0)
	then
		if((select 1 from `groups` where idgroup=idgroup_var and idplatform=idplatform_var)=1)
		then
			if (subgroups_var=1)
			then
				DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups;
				CREATE TEMPORARY TABLE Temp_Table_groups (idgroup int NOT NULL PRIMARY KEY);

				CALL e3tmqtt.Hierarchical_group_subgroups ( idplatform_var, idgroup_var );

				insert Temp_Table_devices
				select D.iddevice, D.token_connection, D.token_connection_date, D.mac, D.status 
				from devices D
				inner join Temp_Table_groups TTG on TTG.idgroup=D.idgroup
				inner join devices_types DT on DT.id=D.idtype
				where 	D.idplatform=idplatform_var
						AND (0=iddevice_type_var OR D.idtype=iddevice_type_var)
						AND (0=iddashboard_type_var OR DT.iddashboard_type=iddashboard_type_var);
			else
				insert Temp_Table_devices
				select D.iddevice, D.token_connection, D.token_connection_date, D.mac, D.status 
				from devices D
				where 	D.idplatform=idplatform_var and D.idgroup = idgroup_var
						AND (0=iddevice_type_var OR D.idtype=iddevice_type_var)
						AND (0=iddashboard_type_var OR DT.iddashboard_type=iddashboard_type_var);
			end if;
		end if;

	elseif (idplatform_var=0 and idgroup_var>0 and iddevice_var=0)
	then
		SET idplatform_var=1;	-- DE MOMENTO DEJAMOS ESTO PARA QUE POR DEFECTO SE MUESTRE LA PLATAFORMA 1
		if (subgroups_var=1)
		then
			DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups;
			CREATE TEMPORARY TABLE Temp_Table_groups (idgroup int NOT NULL PRIMARY KEY);

			CALL e3tmqtt.Hierarchical_group_subgroups ( idplatform_var, idgroup_var );

			insert Temp_Table_devices
			select D.iddevice, D.token_connection, D.token_connection_date, D.mac, D.status 
			from devices D
			inner join Temp_Table_groups TTG on TTG.idgroup=D.idgroup
			inner join devices_types DT on DT.id=D.idtype
			where 	D.idplatform=idplatform_var
					AND (0=iddevice_type_var OR D.idtype=iddevice_type_var)
					AND (0=iddashboard_type_var OR DT.iddashboard_type=iddashboard_type_var);
		else
			insert Temp_Table_devices
			select D.iddevice, D.token_connection, D.token_connection_date, D.mac, D.status 
			from devices D
			inner join devices_types DT on DT.id=D.idtype
			where 	D.idplatform=idplatform_var and D.status=1 and D.idgroup = idgroup_var
					AND (0=iddevice_type_var OR D.idtype=iddevice_type_var)
					AND (0=iddashboard_type_var OR DT.iddashboard_type=iddashboard_type_var);
		end if;

	elseif (idplatform_var>0)
	then
		insert Temp_Table_devices
		select D.iddevice, D.token_connection, D.token_connection_date, D.mac, D.status 
		from devices D
		inner join devices_types DT on DT.id=D.idtype
		where 	D.idplatform=idplatform_var and D.idgroup in (select idgroup from `groups` where idplatform=idplatform_var)
				AND (0=iddevice_type_var OR D.idtype=iddevice_type_var)
				AND (0=iddashboard_type_var OR DT.iddashboard_type=iddashboard_type_var);
	end if;


	IF(		command_var like '%"command":"UPDATE-DEVICE-CONFIG"%' 
		OR 	command_var like '%"command":"DEVICE-MODE"%' OR command_var like '%"command":"DEVICE-STATUS"%'
		OR 	command_var like '%"command":"OUTPUT-MODE"%' OR command_var like '%"command":"OUTPUT-STATUS"%')
	THEN
		UPDATE devices D INNER JOIN Temp_Table_devices TTD ON D.iddevice=TTD.iddevice AND D.idplatform=idplatform_var
		SET D.config_device_date_update=NOW();
	END IF;

	IF(command_var like '%"command":"UPDATE-DEVICE-PARAMS"%')
	THEN
		UPDATE devices D INNER JOIN Temp_Table_devices TTD ON D.iddevice=TTD.iddevice AND D.idplatform=idplatform_var
		SET D.config_device_date_update=NOW(), D.config_params_date_update=NOW();
	END IF;

	IF(command_var like '%"command":"UPDATE-OUTPUT-CONFIG"%')
	THEN
		UPDATE devices D INNER JOIN Temp_Table_devices TTD ON D.iddevice=TTD.iddevice AND D.idplatform=idplatform_var
		SET D.config_output_date_update=NOW();
	END IF;


	-- DELETE FROM Temp_Table_devices WHERE status!=1;


	if (select commands_log from config LIMIT 1)=1
	then
		INSERT devices_commands_log (iddevice, token, token_date, command, date)
		SELECT iddevice, token, token_date, command_var, NOW()
		FROM Temp_Table_devices;
	end if;

	SELECT idplatform_var idplatform, TTD.*, D.idgroup, P.RESTWS_location, P.APPRESTWS_location, D.firmware_sent_date_last, C.MQTT_debug_log, D.secured, D.mqttsn 
	FROM Temp_Table_devices TTD 
	INNER JOIN devices D ON D.iddevice=TTD.iddevice AND D.idplatform=idplatform_var
	INNER JOIN platforms P ON P.id=idplatform_var
	INNER JOIN config C ON 1=1;

end $$
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.WSS_get_command_groups;
$$
create procedure e3tmqtt.WSS_get_command_groups ( idplatform_var INT, idgroup_var INT )
begin
	-- CALL e3tmqtt.WSS_get_command_groups ( 1, 230 );
	SET SQL_SAFE_UPDATES = 0;


	DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups;
	CREATE TEMPORARY TABLE Temp_Table_groups (idgroup int NOT NULL PRIMARY KEY);

	CALL e3tmqtt.Hierarchical_group_subgroups ( idplatform_var, idgroup_var );



	SELECT G.*
	FROM Temp_Table_groups TTG
	INNER JOIN `groups` G ON G.idgroup=TTG.idgroup AND G.idplatform=idplatform_var;

end $$
delimiter ;



drop procedure if exists e3tmqtt.PUSHER_get_device_path_groups;
delimiter $$
create procedure e3tmqtt.PUSHER_get_device_path_groups ( idplatform_var INT, iddevice_var INT )
begin

	-- CALL e3tmqtt.PUSHER_get_device_path_groups ( 1, 125 )

	DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups;
	CREATE TEMPORARY TABLE Temp_Table_groups (id int NOT NULL PRIMARY KEY AUTO_INCREMENT, idgroup int NOT NULL);

	SELECT D.idgroup, DT.iddashboard_type
	FROM devices D
	INNER JOIN devices_types DT ON DT.id=D.idtype
	WHERE D.iddevice=iddevice_var AND D.idplatform=idplatform_var
	LIMIT 1
	INTO @idgroup, @iddashboard_type;

	CALL e3tmqtt.Hierarchical_group_path ( idplatform_var, @idgroup );

	SELECT TTG.*, G.idplatform, @iddashboard_type iddashboard_type 
	FROM Temp_Table_groups TTG 
	INNER JOIN `groups` G ON TTG.idgroup=G.idgroup AND G.idplatform=idplatform_var;

end $$ 
delimiter ;


drop procedure if exists e3tmqtt.PUSHER_get_device_path;
delimiter $$
create procedure e3tmqtt.PUSHER_get_device_path ( idplatform_var INT, iddevice_var INT )
begin

	-- CALL e3tmqtt.PUSHER_get_device_path ( 1, 125 )

	DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups;
	CREATE TEMPORARY TABLE Temp_Table_groups (id int NOT NULL PRIMARY KEY AUTO_INCREMENT, idgroup int NOT NULL);

	SELECT D.idgroup, DT.iddashboard_type
	FROM devices D
	INNER JOIN devices_types DT ON DT.id=D.idtype
	WHERE D.iddevice=iddevice_var AND D.idplatform=idplatform_var
	LIMIT 1
	INTO @idgroup, @iddashboard_type;


	SELECT idplatform_var idplatform, @idgroup idgroup, GROUP_CONCAT(parent.idgroup SEPARATOR '/') group_path, @iddashboard_type iddashboard_type
	FROM `groups` AS node,
			`groups` AS parent
	WHERE node.idplatform=idplatform_var AND parent.idplatform=idplatform_var 
			AND node.lft BETWEEN parent.lft AND parent.rgt
			AND node.idgroup = @idgroup
	ORDER BY node.lft;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.PUSHER_get_platform_groups;
$$
create procedure e3tmqtt.PUSHER_get_platform_groups ( idplatform_var INT )
begin

	-- CALL e3tmqtt.PUSHER_get_platform_groups ( 4 )

	SELECT id idgroup, idplatform FROM `groups` WHERE idplatform=idplatform_var;

end $$ 
delimiter ;



delimiter $$
drop procedure if exists e3tmqtt.WSS_get_service_status;
$$
create procedure e3tmqtt.WSS_get_service_status ( idplatform_var INT, idgroup_var INT, iddevice_var INT, rows_var INT, date_start_var DATE, date_end_var DATE )
begin
	-- CALL e3tmqtt.WSS_get_service_status ( 1, 74, 0, 100, '2015-04-17', '0' )

/*
	select * from devices where idplatform=idplatform_var order by idplatform, idgroup, iddevice;
	select * from `groups` where idplatform=idplatform_var order by idplatform, idgroup;
	select * from devices_commands_log order by id desc limit 100;
	select * from commands_log order by id desc limit 100;
	select * from devices_log order by id desc limit 100;
	select * from messages_log order by id desc limit 100;
	select * from server_log order by id desc limit 100;
*/

	if(iddevice_var = 0)
	then
		set iddevice_var = null;
	end if;


	if(date_start_var!=0)
	then
		SET @subconsulta_fecha_ini=CONCAT('
							AND date >= ''', date_start_var,'''');
	else
		SET @subconsulta_fecha_ini = '';
	end if;

	if(date_end_var!=0)
	then
		SET @subconsulta_fecha_fin=CONCAT('
							AND date <= DATE_ADD(''', date_end_var, ''', INTERVAL 1 DAY)');
	else
		SET @subconsulta_fecha_fin = '';
	end if;

	SET @subconsulta_orden_id=CONCAT('
							ORDER BY id DESC');
	SET @subconsulta_orden_fecha_id=CONCAT('
							ORDER BY fecha DESC, id DESC');


	if(rows_var!=0)
	then
		SET @subconsulta_limit=CONCAT('
							LIMIT ', rows_var);
	else
		SET @subconsulta_limit = '';
	end if;


	DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups;
	CREATE TEMPORARY TABLE Temp_Table_groups (id int NOT NULL PRIMARY KEY AUTO_INCREMENT, idgroup int NOT NULL);

	CALL e3tmqtt.Hierarchical_group_subgroups ( idplatform_var, idgroup_var );

	if iddevice_var is null
	then
		SELECT * 	FROM devices D
		INNER JOIN Temp_Table_groups TTG ON TTG.idgroup=D.idgroup
		WHERE D.idplatform=idplatform_var ORDER BY D.idplatform, D.idgroup, D.iddevice;

		SELECT * 	FROM `groups` G
		INNER JOIN Temp_Table_groups TTG ON TTG.idgroup=G.idgroup
		WHERE G.idplatform=idplatform_var ORDER BY G.idplatform, G.idgroup;

		SET @consulta_base_iddevice= 
			CONCAT(	'SELECT L.* 	FROM #TABLA_LOG# L
					INNER JOIN devices D ON D.iddevice=L.iddevice
					INNER JOIN Temp_Table_groups TTG ON TTG.idgroup=D.idgroup
					WHERE D.idplatform=', idplatform_var, @subconsulta_fecha_ini, @subconsulta_fecha_fin, @subconsulta_orden_id, @subconsulta_limit, ';
				');

		SET @consulta_base_iddevice_idgroup= 
			CONCAT(	'SELECT L.* 	FROM #TABLA_LOG# L
					INNER JOIN `groups` G ON G.idgroup=L.idgroup
					INNER JOIN Temp_Table_groups TTG ON TTG.idgroup=L.idgroup
					WHERE G.idplatform=', idplatform_var, @subconsulta_fecha_ini, @subconsulta_fecha_fin, @subconsulta_orden_id, @subconsulta_limit, ';
				');
	else
		SELECT * 
		FROM devices 
		WHERE iddevice=iddevice_var AND idplatform=idplatform_var;

		SELECT * 
		FROM `groups` G
		INNER JOIN devices D ON D.idgroup=G.idgroup
		WHERE D.iddevice=iddevice_var AND D.idplatform=idplatform_var AND G.idplatform=idplatform_var;

		SET @consulta_base_iddevice= 
			CONCAT(	'SELECT L.* 
					FROM #TABLA_LOG# L
					INNER JOIN devices D ON D.iddevice=L.iddevice
					WHERE D.iddevice=', iddevice_var, ' AND D.idplatform=', idplatform_var, @subconsulta_fecha_ini, @subconsulta_fecha_fin, @subconsulta_orden_id, @subconsulta_limit, ';
				');

		SET @consulta_base_iddevice_idgroup= 
			CONCAT(	'(SELECT L.* 	FROM #TABLA_LOG# L
					LEFT JOIN devices D ON D.iddevice=L.iddevice
					WHERE D.iddevice=', iddevice_var, ' AND D.idplatform=', idplatform_var, @subconsulta_fecha_ini, @subconsulta_fecha_fin, @subconsulta_orden_id, @subconsulta_limit, ')
					UNION
					(SELECT L.* 	FROM #TABLA_LOG# L
					INNER JOIN `groups` G ON G.idgroup=L.idgroup
					INNER JOIN Temp_Table_groups TTG ON TTG.idgroup=L.idgroup
					WHERE (L.iddevice IS NULL OR L.iddevice=0) AND G.idplatform=', idplatform_var, @subconsulta_fecha_ini, @subconsulta_fecha_fin, @subconsulta_orden_id, @subconsulta_limit, ')
					', @subconsulta_orden_id, @subconsulta_limit, ';
				');

	end if;

	SET @consulta_base= CONCAT(	'SELECT L.* 
								FROM #TABLA_LOG# L WHERE 1=1'
								, @subconsulta_fecha_ini, @subconsulta_fecha_fin, @subconsulta_orden_id, @subconsulta_limit, ';
							');

	SELECT * FROM devices_types ORDER BY id;

	SET @consulta=REPLACE(@consulta_base_iddevice, '#TABLA_LOG#', 'devices_commands_log');
	PREPARE smpt FROM @consulta;	EXECUTE smpt;	DEALLOCATE PREPARE smpt;

	SET @consulta=REPLACE(@consulta_base_iddevice_idgroup, '#TABLA_LOG#', 'commands_log');
	PREPARE smpt FROM @consulta;	EXECUTE smpt;	DEALLOCATE PREPARE smpt;

	SET @consulta=REPLACE(@consulta_base_iddevice_idgroup, '#TABLA_LOG#', 'devices_log');
	PREPARE smpt FROM @consulta;	EXECUTE smpt;	DEALLOCATE PREPARE smpt;

	SET @consulta=REPLACE(@consulta_base, '#TABLA_LOG#', 'messages_log');
	PREPARE smpt FROM @consulta;	EXECUTE smpt;	DEALLOCATE PREPARE smpt;

	SET @consulta=REPLACE(@consulta_base, '#TABLA_LOG#', 'server_log');
	PREPARE smpt FROM @consulta;	EXECUTE smpt;	DEALLOCATE PREPARE smpt;

end $$
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.WSS_get_platforms;
$$
create procedure e3tmqtt.WSS_get_platforms ( )
begin

	-- CALL e3tmqtt.WSS_get_platforms ( )

	select * from platforms;

end $$ 
delimiter ;


-- POST

delimiter $$
drop procedure if exists WSS_post_server_log; -- Reconnects Server MQTT
$$
create procedure e3tmqtt.WSS_post_server_log (  )
begin

	-- CALL e3tmqtt.WSS_post_server_log (  )

	SET SQL_SAFE_UPDATES = 0;

	INSERT server_log (date)
	SELECT	NOW();

	set @id = LAST_INSERT_ID();

	INSERT devices_log (idconnection, idplatform, idgroup, iddevice, mac, result, date)
	SELECT	idconnection, idplatform, idgroup, iddevice, mac, -1, NOW()
	FROM devices WHERE status=1;

	UPDATE devices SET status=0, status_date=NOW();

	SELECT @id id;

	-- CALL e3tmqtt.MQTT_historizar_logs();

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.MQTT_historizar_logs;
$$
create procedure e3tmqtt.MQTT_historizar_logs (  )
begin

	-- CALL e3tmqtt.MQTT_historizar_logs (  )

	SET @day_var=(SELECT CONVERT(DATE_ADD(UTC_TIMESTAMP(), INTERVAL -7 DAY), DATE));

	delete from commands_log where date <  @day_var;
	delete from messages_log where date < @day_var;
	delete from devices_commands_log where date < @day_var;
	delete from devices_log where date < @day_var;

	OPTIMIZE TABLE commands_log;
	OPTIMIZE TABLE messages_log;
	OPTIMIZE TABLE devices_commands_log;

	OPTIMIZE TABLE devices;
	OPTIMIZE TABLE `groups`;

end $$ 
delimiter ;

-- ONLY FOR ADDC PROD
/*
delimiter $$
drop procedure if exists e3tmqtt.MQTT_historizar_logs;
$$
create procedure e3tmqtt.MQTT_historizar_logs (  )
begin

	-- CALL e3tmqtt.MQTT_historizar_logs (  )

	SET @day_var=(SELECT CONVERT(DATE_ADD(UTC_TIMESTAMP(), INTERVAL -7 DAY), DATE));

	delete from commands_log where date <  @day_var;
	delete from messages_log where date < @day_var;
	delete from devices_commands_log where date < @day_var;
	delete from devices_log where date < @day_var;
	delete from mqtt_logger_log where TS < @day_var;

	OPTIMIZE TABLE commands_log;
	OPTIMIZE TABLE messages_log;
	OPTIMIZE TABLE devices_commands_log;

	OPTIMIZE TABLE devices;
	OPTIMIZE TABLE `groups`;

	SET @today=date(now());
	SET @yesterday_string=DATE_FORMAT(DATE_SUB(now(), INTERVAL 1 day), "%Y%m%d");
	SET @consulta=CONCAT('ALTER TABLE mqtt_logger_log REORGANIZE PARTITION pMAX INTO ( 
													PARTITION p', @yesterday_string, ' VALUES LESS THAN (''', @today, ''') ENGINE = InnoDB,
													PARTITION pMAX VALUES LESS THAN (MAXVALUE) ENGINE = InnoDB	);');
	PREPARE smpt FROM @consulta;
	EXECUTE smpt;
	DEALLOCATE PREPARE smpt;    

	CALL e3tmqtt.MQTT_logger_log_day ( '0' );

	SET @day_deleted=DATE_FORMAT((SELECT CONVERT(DATE_ADD(UTC_TIMESTAMP(), INTERVAL -8 DAY), DATE)), "%Y%m%d");
	SET @consulta=CONCAT('ALTER TABLE mqtt_logger_log DROP PARTITION p', @day_deleted,';');
	PREPARE smpt FROM @consulta;
	EXECUTE smpt;
	DEALLOCATE PREPARE smpt;    
    

end $$ 
delimiter ;
*/



delimiter $$
drop procedure if exists e3tmqtt.WSS_post_device;
$$
create procedure e3tmqtt.WSS_post_device ( idplatform_var INT, iddevice_var INT, device_XML TEXT )
begin
	/*
	CALL e3tmqtt.WSS_post_device ( 1, 25, '<device>
								<idtype>1</idtype>
								<name>F11, F5 Av Campanar</name>
								<idgroup>7</idgroup>
								<mac>B8-CA-3A-95-86-2C</mac>
								<pw>PBKDF2$sha256$901$fhtxJwfmzs7Rz8cT$htlYr8/1llohIbDGrBpbCqFMOeRR1reZ</pw>
							</device>' )
	*/

	INSERT devices (iddevice, idtype, idgroup, idplatform, name, mac, pw, datalogger_fields, measures_fields, events_fields, mqttsn)
	SELECT	iddevice_var													AS iddevice,
			ExtractValue(device_XML, 'device/idtype') 						AS idtype,
			ExtractValue(device_XML, 'device/idgroup') 						AS idgroup,
			idplatform_var													AS idplatform,
			ExtractValue(device_XML, 'device/name') 						AS name,
			ExtractValue(device_XML, 'device/mac') 							AS mac,
			ExtractValue(device_XML, 'device/pw') 							AS pw,
			datalogger_fields												AS datalogger_fields,
			measures_fields													AS measures_fields,
			events_fields													AS events_fields,
			mqttsn															AS mqttsn
	FROM devices_types WHERE id=ExtractValue(device_XML, 'device/idtype');

	SELECT iddevice_var id;
end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.MQTT_post_device;
$$
create procedure e3tmqtt.MQTT_post_device ( mac_var VARCHAR(45), idplatform_var INT, idgroup_var INT, iddevice_var INT )
begin
	/*
	CALL e3tmqtt.MQTT_post_device ( '11-22-33-44-55', 1, 25, 565 )
	*/

	if (SELECT count(*) FROM devices WHERE mac=mac_var)>0
	then
		UPDATE devices SET idplatform=idplatform_var, idgroup=idgroup_var, iddevice=iddevice_var WHERE mac=mac_var;
	else
		INSERT devices (iddevice, idgroup, idplatform, name, mac)
		SELECT	iddevice_var		AS iddevice,
				idgroup_var 		AS idgroup,
				idplatform_var		AS idplatform,
				mac_var 			AS name,
				mac_var 			AS mac;
	end if;

	SELECT iddevice_var id;
end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.WSS_post_group;
$$
create procedure e3tmqtt.WSS_post_group ( idplatform_var INT, idgroup_var INT, group_XML TEXT )
begin
	/*
	CALL e3tmqtt.WSS_post_group ( 1, 5, '<group>
							<idplatform>1</idplatform>
							<name>Av Maestro Rodrigo</name>
							<parent>2</parent>
						</group>' )
	*/
	SET SQL_SAFE_UPDATES = 0;

	SET @parent_right = (SELECT rgt FROM `groups` WHERE idplatform=idplatform_var AND idgroup = ExtractValue(group_XML, 'group/parent') LIMIT 1);

	UPDATE `groups` SET rgt = rgt + 2 WHERE idplatform=idplatform_var AND rgt >= @parent_right;
	UPDATE `groups` SET lft = lft + 2 WHERE idplatform=idplatform_var AND lft >= @parent_right;

	INSERT `groups` (idgroup, idplatform, name, parent, lft, rgt)
	SELECT	idgroup_var										AS idgroup,
			ExtractValue(group_XML, 'group/idplatform') 	AS idplatform,
			ExtractValue(group_XML, 'group/name') 			AS name,
			ExtractValue(group_XML, 'group/parent') 		AS parent,
			@parent_right									AS lft,
			@parent_right + 1								AS rgt;

	SELECT idgroup_var id;
end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.WSS_post_command_log;
$$
create procedure e3tmqtt.WSS_post_command_log ( command_var VARCHAR(50), idplatform_var INT, idgroup_var INT, iddevice_var INT, parameters_var TEXT, request_var TEXT )
begin
	-- CALL e3tmqtt.WSS_post_command_log ( 'DEVICE-STATUS', '1', '1', '1', '{"idoutput_type":"1","iddevice_type":"1","output_number":"1","idmode":"1","subgroups":"1"}', '{"command":"OUTPUT-MODE","idplatform":"1","idgroup":"1","iddevice":"0","parameters":{"idoutput_type":"1","iddevice_type":"1","output_number":"1","idmode":"1","subgroups":"1"},"when":1416394870}' )


	if (select commands_log from config LIMIT 1)=1
	then
		INSERT commands_log (idplatform, idgroup, iddevice, command, parameters, request, date)
		SELECT	idplatform_var, idgroup_var, iddevice_var, command_var, parameters_var, request_var, NOW();

		SELECT LAST_INSERT_ID() id;
	else
		select 1 id;
	end if;

end $$ 
delimiter ;

delimiter $$
drop procedure if exists e3tmqtt.WSS_post_device_log;
$$
create procedure e3tmqtt.WSS_post_device_log ( token_var VARCHAR(50), date_mills_var BIGINT, idplatform_var INT, idgroup_var INT, iddevice_var INT, mac_var VARCHAR(45), result_var INT )
begin

	-- CALL e3tmqtt.WSS_post_device_log ( 1, 2, 1, '21-21-21-21-21', 1 )

	if result_var!=0
	then
		update devices set status=result_var, status_date=NOW(), token_connection=token_var, token_connection_date=date_mills_var 
		where idplatform=idplatform_var and idgroup=idgroup_var and iddevice=iddevice_var;
	end if;

	if (select devices_log from config LIMIT 1)=1
	then
		INSERT devices_log (token, token_date, idplatform, idgroup, iddevice, mac, result, date)
		SELECT	token_var, date_mills_var, idplatform_var, idgroup_var, iddevice_var, mac_var, result_var, NOW();
	end if;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.WSS_post_message_log;
$$
create procedure e3tmqtt.WSS_post_message_log ( idconnection_var INT, remoteAddress_var VARCHAR(200), topic_var VARCHAR(450), message_var VARCHAR(5000), idplatform_var INT, idgroup_var INT, iddevice_var INT )
begin
	-- CALL e3tmqtt.WSS_post_message_log ( 49, '87.216.176.159', '/00-11-22-33-44-55/rx', '{"device":1, "idplatform":1, "idgroup":2, "iddevice":1, "mac":"21-21-21-21-21"}', '1', '2', '1' )


	if (select messages_log from config LIMIT 1)=1
	then
		INSERT messages_log (idconnection, remoteAddress, topic, message, idplatform, idgroup, iddevice, date)
		SELECT 	idconnection_var, remoteAddress_var, topic_var, message_var, idplatform_var, 
				IF(idgroup_var is null OR idgroup_var=0,(SELECT idgroup FROM devices WHERE iddevice=iddevice_var AND idplatform=idplatform_var), idgroup_var),
				iddevice_var, NOW();

		SELECT LAST_INSERT_ID() id;
	else
		select 1 id;
	end if;

end $$ 
delimiter ;



-- PUT


delimiter $$
drop procedure if exists e3tmqtt.WSS_put_device;
$$
create procedure e3tmqtt.WSS_put_device ( idplatform_var INT, iddevice_var INT, device_XML TEXT )
begin
	/*
	CALL e3tmqtt.WSS_put_device ( 1, 2637, '<device>
					<idtype>40</idtype>
					<name>OP00050F241017C000001</name>
					<idgroup>261</idgroup>
					<mac>867567021329186</mac>
					<pw>PBKDF2$sha256$901$8NLK8tzUYSo6pou6$pGIb04Mif7XIoMx1JqRRQHp4QPiXiNdg</pw>
				</device>' )
	*/

	SELECT idtype, ExtractValue(device_XML, 'device/idtype') FROM devices WHERE idplatform=idplatform_var AND iddevice=iddevice_var
	INTO @idtype, @idtype_new;

	UPDATE devices D
	INNER JOIN devices_types DT ON DT.id=ExtractValue(device_XML, 'device/idtype')
				SET 	D.idtype = 				ExtractValue(device_XML, 'device/idtype'),
						D.idgroup = 			ExtractValue(device_XML, 'device/idgroup'),
						D.name = 				ExtractValue(device_XML, 'device/name'),
						D.mac = 				ExtractValue(device_XML, 'device/mac'),
						D.pw = 					ExtractValue(device_XML, 'device/pw'),
						D.datalogger_fields = 	IF(@idtype=@idtype_new, D.datalogger_fields, DT.datalogger_fields),
						D.measures_fields =		IF(@idtype=@idtype_new, D.measures_fields, DT.measures_fields),
						D.events_fields =		IF(@idtype=@idtype_new, D.events_fields, DT.events_fields),
						D.config_serial_response = IF(@idtype=@idtype_new, D.config_serial_response, DT.config_serial_response),
						D.mqttsn =				IF(@idtype=@idtype_new, D.mqttsn, DT.mqttsn),
						D.config_device_request_allways =				IF(@idtype=@idtype_new, D.config_device_request_allways, DT.config_device_request_allways),
						D.config_params_request_allways =				IF(@idtype=@idtype_new, D.config_params_request_allways, DT.config_params_request_allways),
						D.config_output_request_allways =				IF(@idtype=@idtype_new, D.config_output_request_allways, DT.config_output_request_allways)
	WHERE D.idplatform=idplatform_var AND D.iddevice=iddevice_var;

	if FOUND_ROWS()=0
	then
		CALL e3tmqtt.WSS_post_device ( idplatform_var, iddevice_var, device_XML );
	else
		SELECT iddevice_var id;
	end if;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.WSS_put_devices_group;
$$
create procedure e3tmqtt.WSS_put_devices_group ( idplatform_var INT, devices_group_XML TEXT )
begin
	/*
	CALL e3tmqtt.WSS_put_devices_group ( 4, '<devices_group>
								<idgroup>3</idgroup>
								<iddevices>2</iddevices>
							</devices_group>' )
	*/


	SET @consulta = CONCAT('UPDATE devices SET 	idgroup = ', ExtractValue(devices_group_XML, 'devices_group/idgroup'),
					' WHERE idplatform = ', idplatform_var, 
					' AND iddevice IN ( ', ExtractValue(devices_group_XML, 'devices_group/iddevices'), ' )');

	-- preparamos el objete Statement a partir de nuestra variable
	PREPARE smpt FROM @consulta;
	-- ejecutamos el Statement
	EXECUTE smpt;

	select ROW_COUNT() result;

	-- liberamos la memoria
	DEALLOCATE PREPARE smpt;

end $$
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.WSS_put_devices_type;
$$
create procedure e3tmqtt.WSS_put_devices_type ( idplatform_var INT, devices_group_XML TEXT )
begin
	/*
	CALL e3tmqtt.WSS_put_devices_type ( 4, '<devices_type>
								<idtype>3</idtype>
								<iddevices>2</iddevices>
							</devices_type>' )
	*/


	SET @consulta = CONCAT('UPDATE devices SET 	idtype = ', ExtractValue(devices_group_XML, 'devices_group/idtype'),
					' WHERE idplatform = ', idplatform_var, 
					' AND iddevice IN ( ', ExtractValue(devices_group_XML, 'devices_group/iddevices'), ' )');

	-- preparamos el objete Statement a partir de nuestra variable
	PREPARE smpt FROM @consulta;
	-- ejecutamos el Statement
	EXECUTE smpt;

	select ROW_COUNT() result;

	-- liberamos la memoria
	DEALLOCATE PREPARE smpt;

end $$
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.WSS_put_group;
$$
create procedure e3tmqtt.WSS_put_group ( idplatform_var INT, idgroup_var INT, group_XML TEXT )
begin
	/*
	CALL e3tmqtt.WSS_put_group ( 1, 8, '<group>
							<name>Av Maestro Rodrigo</name>
							<parent>2</parent>
						</group>' )
	*/

	SET @parent_ini = (select parent from `groups` where idplatform=idplatform_var AND idgroup=idgroup_var LIMIT 1);

	UPDATE `groups` SET 	name = 				ExtractValue(group_XML, 'group/name'),
						parent = 	ExtractValue(group_XML, 'group/parent')
	WHERE idplatform=idplatform_var AND idgroup=idgroup_var;

	if FOUND_ROWS()=0
	then
		CALL e3tmqtt.WSS_post_group ( idplatform_var, idgroup_var, group_XML );
	else
		if @parent_ini != ExtractValue(group_XML, 'group/parent')
		then
			CALL e3tmqtt.Hierarchical_groups_regenerate_all ( idplatform_var );
		end if;

		SELECT idgroup_var id;
	end if;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.MQTT_put_device_firmware_sent_date_last;
$$
create procedure e3tmqtt.MQTT_put_device_firmware_sent_date_last ( idplatform_var INT, iddevice_var INT )
begin
	/*
	CALL e3tmqtt.MQTT_put_device_firmware_sent_date_last ( 1, 8 )
	*/

	UPDATE devices SET 	firmware_sent_date_last = NOW()
	WHERE idplatform=idplatform_var AND iddevice=iddevice_var;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.MQTT_put_device_config_device;
$$
create procedure e3tmqtt.MQTT_put_device_config_device ( idplatform_var INT, iddevice_var INT, config_var TEXT )
begin
	/*
	CALL e3tmqtt.MQTT_put_device_config_device ( 1, 8, '[{"idmode":"0","idstatus":"1","timezone_offset":"1","timezone_dst":"1","timezone_dst_date_next_start":"2018-03-25 01:00:00","timezone_dst_date_next_end":"2017-10-29 01:00:00","wifis":[{"wifi1_name":"MOVISTAR_740B","wifi1_pass":"48182713"},{"wifi2_name":"Orange-173C","wifi2_pass":"AE27AF65"},{"wifi3_name":"e3tcity","wifi3_pass":"e3tcitye3t"}],"iddevice_type":"40","version_firmware_last":"FW800009_PCBA_CPU_LP10_V01_01","data_logger_idmode":"1","WSS_location":"http://ec2-54-194-201-140.eu-west-1.compute.amazonaws.com:9311","RESTWS_location":"http://e3tapp.com/restws","GPS_lat":"39.47243173612800000000","GPS_lng":"-0.38953729462582000000","server":"172.31.9.235","port":"1883","keepalive":"60"}]' )
	*/

	SET @version_firmware_last= (SELECT
		DISTINCT(substring(config_var, locate('"version_firmware_last":',config_var)+LENGTH('"version_firmware_last":')+1,
			locate('","', config_var, locate('"version_firmware_last":',config_var))-locate('"version_firmware_last":',config_var)-(LENGTH('"version_firmware_last":')+1))) );

	UPDATE devices SET version_firmware_last = @version_firmware_last, config_device_date = NOW(), config_device=config_var
	WHERE idplatform=idplatform_var AND iddevice=iddevice_var;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.MQTT_put_device_config_params;
$$
create procedure e3tmqtt.MQTT_put_device_config_params ( idplatform_var INT, iddevice_var INT, config_var TEXT )
begin
	/*
	CALL e3tmqtt.MQTT_put_device_config_params ( 1, 8, '{"apn_name":"e3tcity.movistar.es","apn_user":"MOVISTAR","apn_pass":"MOVISTAR","apn_title":"MOVISTAR","keepalive":"600","dl_freq_calc":"3","dl_freq_send":"0","dl_calc_send":"3","meter_type":"0","meter_params":[{"voltage":[{"enable":"0","step":"5"},{"enable":"0","step":"5"},{"enable":"0","step":"5"},{"enable":"0","step":"5"}],"current":[{"enable":"0","step":"1"},{"enable":"0","step":"1"},{"enable":"0","step":"1"},{"enable":"0","step":"1"}],"poweractive":[{"enable":"0","step":"250"},{"enable":"0","step":"250"},{"enable":"0","step":"250"},{"enable":"0","step":"250"}],"powerreactive":[{"enable":"0","step":"1"},{"enable":"0","step":"1"},{"enable":"0","step":"1"},{"enable":"0","step":"1"}],"energyactive":{"0":{"enable":"0","step":"1"},"3ph":{"enable":"0","step":"1"}},"energyreactive":{"0":{"enable":"0","step":"1"},"3ph":{"enable":"0","step":"1"}}}],"inputs":[{"n":"1","enable":"0","type":"0","min":"0","step":"10","max":"2500","mcc":"3"},{"n":"2","enable":"0","type":"0","min":"0","step":"10","max":"2500","mcc":"3"},{"n":"3","enable":"0","type":"0","min":"0","step":"10","max":"2500","mcc":"3"},{"n":"4","enable":"0","type":"0","min":"0","step":"10","max":"2500","mcc":"3"},{"n":"5","enable":"0","type":"0","min":"0","step":"10","max":"2500","mcc":"3"},{"n":"6","enable":"0","type":"0","min":"0","step":"10","max":"2500","mcc":"3"}],"outputs":[{"n":"1","type":"0"}],"events":[{"n":"1","enable":"1","type":"2","sensor":"2","p1":"1","p2":"1","p3":"0","p4":"0","p5":"0"},{"n":"2","enable":"1","type":"2","sensor":"3","p1":"4","p2":"1","p3":"0","p4":"0","p5":"0"},{"n":"3","enable":"0","type":"0","sensor":"0","p1":"0","p2":"0","p3":"0","p4":"0","p5":"0"},{"n":"4","enable":"0","type":"0","sensor":"0","p1":"0","p2":"0","p3":"0","p4":"0","p5":"0"},{"n":"5","enable":"0","type":"0","sensor":"0","p1":"0","p2":"0","p3":"0","p4":"0","p5":"0"},{"n":"6","enable":"0","type":"0","sensor":"0","p1":"0","p2":"0","p3":"0","p4":"0","p5":"0"}]}' )
	*/

	UPDATE devices SET 	config_params_date = NOW(), config_params=config_var
	WHERE idplatform=idplatform_var AND iddevice=iddevice_var;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.MQTT_put_device_config_output;
$$
create procedure e3tmqtt.MQTT_put_device_config_output ( idplatform_var INT, iddevice_var INT, config_var TEXT )
begin
	/*
	CALL e3tmqtt.MQTT_put_device_config_output ( 1, 8, '[]' )
	*/

	UPDATE devices SET 	config_output_date = NOW(), config_output=config_var
	WHERE idplatform=idplatform_var AND iddevice=iddevice_var;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.MQTT_put_device_mqtt;
$$
create procedure e3tmqtt.MQTT_put_device_mqtt ( idplatform_var INT, iddevice_var INT )
begin
	/*
	CALL e3tmqtt.MQTT_put_device_mqtt ( 1, 8, '[]' )
	*/
	UPDATE devices SET 	connection_hhwid=0, connection_device=1
	WHERE idplatform=idplatform_var AND idgroup=idgroup_var AND iddevice=iddevice_var;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.MQTT_put_device_mqtt_hwid;
$$
create procedure e3tmqtt.MQTT_put_device_mqtt_hwid ( mac_var VARCHAR(45), RESTWS_location_var VARCHAR(450) )
begin
	/*
	CALL e3tmqtt.MQTT_put_device_mqtt_hwid ( 1, 8, '[]' )
	*/
	UPDATE devices SET 	connection_hhwid=1, connection_device=0, RESTWS_location = RESTWS_location_var
	WHERE mac=mac_var;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.MQTT_put_device_fields_reset;
$$
create procedure e3tmqtt.MQTT_put_device_fields_reset ( idplatform_var INT, iddevice_var INT )
begin
	
	-- CALL e3tmqtt.MQTT_put_device_fields_reset ( 1, 2589 )

	UPDATE devices D
	INNER JOIN devices_types DT ON DT.id = D.idtype
	SET D.events_fields=DT.events_fields, D.datalogger_fields=DT.datalogger_fields
	WHERE  D.iddevice=iddevice_var AND D.idplatform=idplatform_var; 

end $$ 
delimiter ;

delimiter $$
drop procedure if exists e3tmqtt.MQTT_put_device_events_fields;
$$
create procedure e3tmqtt.MQTT_put_device_events_fields ( idplatform_var INT, iddevice_var INT, events_fields_var VARCHAR(2000) )
begin
	
	-- CALL e3tmqtt.MQTT_put_device_events_fields ( 1, 2589, 'date;idtrigger_type;idmode;idstatus;dimmer;power_active;current;input1;input2;input3;input4' )

	UPDATE devices 
	SET events_fields=events_fields_var
	WHERE  iddevice=iddevice_var AND idplatform=idplatform_var; 

end $$ 
delimiter ;

delimiter $$
drop procedure if exists e3tmqtt.MQTT_put_device_datalogger_fields;
$$
create procedure e3tmqtt.MQTT_put_device_datalogger_fields ( idplatform_var INT, iddevice_var INT, datalogger_fields_var VARCHAR(4000) )
begin
	
	-- CALL e3tmqtt.MQTT_put_device_datalogger_fields ( 1, 2589, 'date, type, energy, switching_on, switch_on_time, connected_time, disconnections, power_active_avg, power_active_min, power_active_max, current_avg, current_min, current_max, input1_avg, input1_min, input1_max, input2_avg, input2_min, input2_max, input3_avg, input3_min, input3_max, input4_avg, input4_min, input4_max' )

	UPDATE devices 
	SET datalogger_fields=datalogger_fields_var
	WHERE  iddevice=iddevice_var AND idplatform=idplatform_var;  

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.APP_put_devices_replacement;
$$
create procedure e3tmqtt.APP_put_devices_replacement( idplatform_var INT, iddevice_old_var INT, iddevice_new_var INT )
begin
	-- CALL APP_put_devices_replacement (1, 2819, 2497);

	SELECT mac FROM devices WHERE idplatform=idplatform_var AND iddevice=iddevice_old_var
	INTO @device_old_mac;

	SELECT mac FROM devices WHERE idplatform=idplatform_var AND iddevice=iddevice_new_var
	INTO @device_new_mac;

	INSERT devices_replacement_log
	(date, idplatform, device_old_iddevice, device_old_mac, device_new_iddevice, device_new_mac)
	SELECT NOW(), idplatform_var, iddevice_old_var, @device_old_mac, iddevice_new_var, @device_new_mac;

	UPDATE devices 
	SET 	mac = CONCAT('XXX', @device_new_mac)
	WHERE idplatform=idplatform_var AND iddevice=iddevice_old_var;

	UPDATE devices 
	SET 	mac = @device_old_mac
	WHERE idplatform=idplatform_var AND iddevice=iddevice_new_var;

	UPDATE devices 
	SET 	mac = @device_new_mac
	WHERE idplatform=idplatform_var AND iddevice=iddevice_old_var;

end$$



-- DELETE


delimiter $$
drop procedure if exists e3tmqtt.WSS_delete_device;
$$
create procedure e3tmqtt.WSS_delete_device ( idplatform_var INT, iddevice_var INT)
begin
	-- CALL e3tmqtt.WSS_delete_device ( 1, 21 )

	DELETE FROM devices WHERE idplatform=idplatform_var AND iddevice=iddevice_var;

	if ROW_COUNT()>0
	then
		select 1 result;
	else
		select 0 result;
	end if;
end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.WSS_delete_group;
$$
create procedure e3tmqtt.WSS_delete_group ( idplatform_var INT, idgroup_var INT)
begin
	-- CALL e3tmqtt.WSS_delete_group ( 1, 1 )
	SET SQL_SAFE_UPDATES = 0;

	SELECT lft, rgt, rgt - lft + 1, parent
	FROM `groups`
	WHERE idplatform=idplatform_var AND idgroup = idgroup_var
	INTO @group_left, @group_right, @group_width, @parent;

--  PARA ELIMINAR TODOS LOS GRUPOS BAJO ESTE
-- 	DELETE FROM `groups` WHERE idplatform=idplatform_var AND lft BETWEEN @group_left AND @group_right;

--  PARA ELIMINAR SOLO ESTE GRUPO
	DELETE FROM `groups` WHERE idplatform=idplatform_var AND idgroup=idgroup_var;

	if ROW_COUNT()>0
	then
		select 1 result;
	else
		select 0 result;
	end if;

--  PARA ELIMINAR TODOS LOS GRUPOS BAJO ESTE
-- 	UPDATE `groups` SET rgt = rgt - @group_width WHERE idplatform=idplatform_var AND rgt > @group_right;
-- 	UPDATE `groups` SET lft = lft - @group_width WHERE idplatform=idplatform_var AND lft > @group_right;

--  PARA PROMOCIONAR LOS GRUPOS BAJO EL ELIMINADO ( SOLO UNO )
	UPDATE devices SET idgroup=@parent WHERE idplatform=idplatform_var AND idgroup=idgroup_var;
	UPDATE `groups` SET parent=@parent WHERE idplatform=idplatform_var AND parent=idgroup_var;

	UPDATE `groups` SET rgt = rgt - 1, lft = lft - 1 WHERE idplatform=idplatform_var AND lft BETWEEN @group_left AND @group_right;
	UPDATE `groups` SET rgt = rgt - 2 WHERE idplatform=idplatform_var AND rgt > @group_right;
	UPDATE `groups` SET lft = lft - 2 WHERE idplatform=idplatform_var AND lft > @group_right;

end $$ 
delimiter ;










-- TABLAS JERARQUICAS

delimiter $$
drop procedure if exists e3tmqtt.Hierarchical_group_subgroups;
$$
create procedure e3tmqtt.Hierarchical_group_subgroups ( idplatform_var INT, idgroup_var INT )
begin
	-- CALL e3tmqtt.Hierarchical_group_subgroups ( 1, 66 )
	-- SELECT * FROM Temp_Table_groups;

	DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups;
	CREATE TEMPORARY TABLE IF NOT EXISTS Temp_Table_groups (idgroup int NOT NULL PRIMARY KEY);


	-- SUBNIVELES BAJO UN SUBNIVEL
	INSERT Temp_Table_groups (idgroup)
	SELECT node.idgroup idgroup
	FROM `groups` AS node,
			`groups` AS parent,
			`groups` AS sub_parent,
			(
					SELECT node.idgroup, (COUNT(parent.idgroup) - 1) AS depth
					FROM `groups` AS node,
							`groups` AS parent
					WHERE  node.idplatform=idplatform_var AND parent.idplatform=idplatform_var 
							AND node.lft BETWEEN parent.lft AND parent.rgt
							AND node.idgroup = idgroup_var
					GROUP BY node.idgroup
					ORDER BY node.lft
			)AS sub_tree
	WHERE node.idplatform=idplatform_var AND parent.idplatform=idplatform_var 
		AND node.lft BETWEEN parent.lft AND parent.rgt
			AND node.lft BETWEEN sub_parent.lft AND sub_parent.rgt
			AND sub_parent.idgroup = sub_tree.idgroup
	GROUP BY node.idgroup
	-- PARA VER SOLO LOS SUBNIVELES DIRECTOS
	-- HAVING depth <= 1
	-- PARA NO VER EL NIVEL:
	-- HAVING idgroup != idgroup_var
	ORDER BY node.lft;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.Hierarchical_group_path;
$$
create procedure e3tmqtt.Hierarchical_group_path ( idplatform_var INT, idgroup_var INT )
begin
	-- CALL e3tmqtt.Hierarchical_group_path ( 1, 3 )

	CREATE TEMPORARY TABLE IF NOT EXISTS Temp_Table_groups (id int NOT NULL PRIMARY KEY AUTO_INCREMENT, idgroup int NOT NULL);

	-- RUTA COMPLETA DE UN GRUPO
	INSERT Temp_Table_groups (idgroup)
	SELECT parent.idgroup
	FROM `groups` AS node,
			`groups` AS parent
	WHERE node.idplatform=idplatform_var AND parent.idplatform=idplatform_var 
		AND node.lft BETWEEN parent.lft AND parent.rgt
		AND node.idgroup = idgroup_var
	ORDER BY node.lft;
	
	-- SELECT * FROM Temp_Table_groups;
	-- drop table Temp_Table_groups;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.Hierarchical_groups_regenerate_all;
$$
create procedure e3tmqtt.Hierarchical_groups_regenerate_all (idplatform_var INT)
begin
	-- CALL e3tmqtt.Hierarchical_groups_regenerate_all (1)

	SET SQL_SAFE_UPDATES = 0;
	
	set @parent=1;
	set @id=0;

	DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups;
	CREATE TEMPORARY TABLE IF NOT EXISTS Temp_Table_groups (idgroup int NOT NULL PRIMARY KEY, parent int NULL, lft int NULL, rgt int NULL);
	DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups_processed;
	CREATE TEMPORARY TABLE IF NOT EXISTS Temp_Table_groups_processed (id int NOT NULL PRIMARY KEY AUTO_INCREMENT, idgroup int NOT NULL);
	DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups_level;
	CREATE TEMPORARY TABLE IF NOT EXISTS Temp_Table_groups_level (id int NOT NULL PRIMARY KEY AUTO_INCREMENT, idgroup int NOT NULL);

	INSERT Temp_Table_groups (idgroup, parent, lft, rgt) SELECT idgroup, parent, lft, rgt FROM `groups` WHERE idplatform=idplatform_var;

	update Temp_Table_groups set lft=null, rgt=null;
	update Temp_Table_groups set lft=1,rgt=2 where idgroup=1 and parent is null;

	INSERT Temp_Table_groups_processed (idgroup) SELECT 1 idgroup;
	INSERT Temp_Table_groups_level (idgroup) SELECT idgroup FROM `groups` WHERE idplatform=idplatform_var and parent=1;

	while @parent is not null
	do
		set @parent_rgt = (select rgt from Temp_Table_groups where idgroup=@parent);

		while @parent is not null and @id is not null
		do
			-- select 'SON' TYPE, `groups`.* from `groups` where id>@id and parent=@parent order by parent, id limit 1;

			set @id = (select idgroup from Temp_Table_groups where idgroup>@id and parent=@parent order by parent, idgroup limit 1);

			if @id is not null
			then
				-- llamamos a insertar esta categoria bajo el parent
				UPDATE Temp_Table_groups SET rgt = rgt + 2 WHERE rgt >= @parent_rgt;
				UPDATE Temp_Table_groups SET lft = lft + 2 WHERE lft >= @parent_rgt;

				UPDATE Temp_Table_groups SET lft = @parent_rgt, rgt = @parent_rgt + 1 WHERE idgroup=@id;

				set @parent_rgt = (select rgt from Temp_Table_groups where idgroup=@parent);

				INSERT Temp_Table_groups_processed (idgroup) SELECT @id idgroup;
			end if;

		end while;

		set @id=0;

		-- select 'PARENT' TYPE, `groups`.* from `groups` where id not in (select idgroup from Temp_Table_groups_processed) 
		-- 											and id in (select idgroup from Temp_Table_groups_level) order by parent, id limit 1;

		set @parent = (select parent from Temp_Table_groups where idgroup not in (select idgroup from Temp_Table_groups_processed) 
														and idgroup in (select idgroup from Temp_Table_groups_level) order by parent, idgroup limit 1);
		IF(@parent is null)
		THEN
			TRUNCATE TABLE Temp_Table_groups_level;
			INSERT Temp_Table_groups_level (idgroup)
			SELECT idgroup FROM Temp_Table_groups WHERE lft is null AND parent IN (SELECT idgroup FROM Temp_Table_groups_processed);

			-- select 'PARENT' TYPE, `groups`.* from `groups` where id not in (select idgroup from Temp_Table_groups_processed) 
			-- 												and id in (select idgroup from Temp_Table_groups_level) order by parent, id limit 1;

			set @parent = (select parent from Temp_Table_groups where idgroup not in (select idgroup from Temp_Table_groups_processed) 
															and idgroup in (select idgroup from Temp_Table_groups_level) order by parent, idgroup limit 1);
		END IF;

	end while;

    -- ACTUALIZAMOS LOS VALORES DE LA TABLA GRUPOS
    UPDATE `groups` G INNER JOIN Temp_Table_groups TTG ON TTG.idgroup=G.idgroup AND G.idplatform=idplatform_var SET G.lft=TTG.lft, G.rgt=TTG.rgt;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.ADMIN_clear_logs;
$$
create procedure e3tmqtt.ADMIN_clear_logs (  )
begin
	-- CALL e3tmqtt.ADMIN_clear_logs ( )

	delete from commands_log where date<DATE_FORMAT(NOW(), '%Y-01-01');
	delete from devices_commands_log where date<DATE_FORMAT(NOW(), '%Y-01-01');
	delete from devices_log where date<DATE_FORMAT(NOW(), '%Y-01-01');
	delete from messages_log where date<DATE_FORMAT(NOW(), '%Y-01-01');
	delete from server_log where date<DATE_FORMAT(NOW(), '%Y-01-01');

end $$ 
delimiter ;







-- PROCEDIMIENTOS MQTT AUTENTICACION Y HOOKS

delimiter $$
drop procedure if exists e3tmqtt.MQTT_AUTH_auth;
$$
create procedure e3tmqtt.MQTT_AUTH_auth ( username_var VARCHAR(200), password_var VARCHAR(200), user_pass_var VARCHAR(200), user_mac_imei_var VARCHAR(200), user_counter_var VARCHAR(200), user_random_var VARCHAR(200), request_var VARCHAR(2000) )
begin
	-- CALL e3tmqtt.MQTT_AUTH_auth ( '25-32-47-8A-7B_1214783', '6099815CE544B088B8F04D791F1347FC', '6099815CE544B088B8F04D791F1347FC', '25-32-47-8A-7B', '121', '4783' );
	-- CALL e3tmqtt.MQTT_AUTH_auth ( 'e3t.MQTT.CMDREST', 'PBKDF2$sha256$901$SPCW2NbWwYdk44fC$OrViUitS2dcPWsi9iYinQbr+gw0rHyDU', '6099815CE544B088B8F04D791F1347FC', '25-32-47-8A-7B', '121', '4783' );
	-- CALL e3tmqtt.MQTT_AUTH_auth ( '356345080030611_2024783', 'B562AFC48442310B532F6DD011F9CF6A', 'B562AFC48442310B532F6DD011F9CF6A', '356345080030611', '202', '4783' );
	-- CALL e3tmqtt.MQTT_AUTH_auth ( 'APP.User_154832071234167248', 'D89C5C68D20826992DEAC2CA36BBBEDC', 'D89C5C68D20826992DEAC2CA36BBBEDC', 'APP.User', '15483207123416', '7248' );

	SELECT 0, null, null, null INTO @result, @iduser, @device, @idplatform;

	IF LOCATE('_@P@', user_mac_imei_var)
	THEN
		SET @user = SUBSTRING(user_mac_imei_var, 1, LOCATE('_@P@', user_mac_imei_var)-1);
	ELSE
		SET @user = user_mac_imei_var;
	END IF;

	SELECT 1 result, iduser, device, idplatform FROM 
	(	SELECT 1 result, id iduser, 0 device, 0 idplatform FROM MQTT_users WHERE user=@user AND password_var=user_pass_var AND idconnection_last < user_counter_var
		UNION
		SELECT 1 result, iddevice iduser, 1 device, idplatform FROM devices WHERE mac=user_mac_imei_var AND password_var=user_pass_var AND idconnection < user_counter_var
	) as USERS order by result DESC limit 1
	INTO @result, @iduser, @device, @idplatform;

	if(@result=1)
	then
		if @device=0
		then
			UPDATE MQTT_users SET pass=password_var, idconnection_last=user_counter_var WHERE id=@iduser;
		end if;
		if @device=1
		then
			UPDATE devices SET pw=password_var, idconnection=user_counter_var WHERE idplatform=@idplatform AND iddevice=@iduser;
		end if;
	end if;

	INSERT MQTT_AUTH_auth_log (date, result, iduser, username, password, user_pass, user_mac_imei, user_counter, user_random, request)
	SELECT NOW(), @result, @iduser, username_var, password_var, user_pass_var, user_mac_imei_var, user_counter_var, user_random_var, request_var;
	
	SELECT @result result, @iduser iduser;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.MQTT_AUTH_superuser;
$$
create procedure e3tmqtt.MQTT_AUTH_superuser ( username_var VARCHAR(200), request_var VARCHAR(2000) )
begin
	-- CALL e3tmqtt.MQTT_AUTH_superuser ( '25-32-47-8A-7B_1214783' );
	-- CALL e3tmqtt.MQTT_AUTH_superuser ( 'e3t.MQTT.CMDREST' );
	-- CALL e3tmqtt.MQTT_AUTH_superuser ( '356345080030611_1214783' );

	SELECT COUNT(superuser) result, id iduser FROM MQTT_users WHERE user=username_var AND superuser=1 limit 1
	INTO @result, @iduser;

	INSERT MQTT_AUTH_superuser_log (date, iduser, username, result, request)
	SELECT NOW(), @iduser iduser, username_var, @result result, request_var;

	SELECT @result result, @iduser iduser;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.MQTT_AUTH_acl;
$$
create procedure e3tmqtt.MQTT_AUTH_acl ( username_var VARCHAR(200), clientid_var VARCHAR(200), topic_var VARCHAR(200), acc_var VARCHAR(20), request_var VARCHAR(2000))
begin
	-- CALL e3tmqtt.MQTT_AUTH_acl ( '25-32-47-8A-7B_1214783', '25-32-47-8A-7B', '/device/1/25/ans/#', '1' );
	-- CALL e3tmqtt.MQTT_AUTH_acl ( 'e3t.MQTT.CMDREST', 'e3t.MQTT.CMDREST', '/device/1/25/ans/#', '1' );
	-- CALL e3tmqtt.MQTT_AUTH_acl ( '356345080030611_1214783', '356345080030611', '/device/7/508003061/set/#', '1' );
	-- CALL e3tmqtt.MQTT_AUTH_acl ( 'APP.User_@P@7_@G@366_154955496627537236', 'client id not available', '/PUSHER/@P@7/@G@/1/366', '4', '{"username":"APP.User_@P@7_@G@366_154955496627537236","password":"","topic":"\/PUSHER\/@P@7\/@G@\/1\/366","acc":"4","clientid":"client id not available"}');

	if clientid_var = 'client id not available'
	then
		set @mac_imei = SUBSTRING(username_var, 1, CHAR_LENGTH(username_var) - LOCATE('_', REVERSE(username_var)));
	else
		set @mac_imei = clientid_var;
	end if;

	IF LOCATE('_@P@', @mac_imei)
	THEN
		SET @user = SUBSTRING(@mac_imei, 1, LOCATE('_@P@', @mac_imei)-1);
	ELSE
		SET @user = @mac_imei;
	END IF;

	IF LOCATE('_@P@', @mac_imei)
	THEN
		SELECT null, null, null INTO @idplatform, @idgroup, @iddevice;

		SET @user_aux = SUBSTRING( @user, LOCATE('_@P@', @mac_imei)+1 );
		-- SET @user = SUBSTRING( @user, 1, LOCATE('_@P@', @user)-1 );
		SET @idplatform = SUBSTRING( @user_aux, (LOCATE('@P@', @user_aux)+3), IF(LOCATE('_', @user_aux), LOCATE('_', @user_aux)-4, LENGTH(@user_aux)) );
		SET @user_aux = SUBSTRING( @user_aux, LOCATE('_', @user_aux)+1 );

		if LOCATE('@G@', @user_aux)
		then 
			SET @idgroup = SUBSTRING( @user_aux, (LOCATE('@G@', @user_aux)+3), IF(LOCATE('_', @user_aux), LOCATE('_', @user_aux)-4, LENGTH(@user_aux)) );
			SET @user_aux = SUBSTRING( @user_aux, LOCATE('_', @user_aux)+1 );
		end if;
		if LOCATE('@D@', @user_aux)
		then 
			SET @iddevice = SUBSTRING( @user_aux, (LOCATE('@D@', @user_aux)+3), IF(LOCATE('_', @user_aux), LOCATE('_', @user_aux)-4, LENGTH(@user_aux)) );
		end if;

		IF LOCATE('/@P@', topic_var)
		THEN
			SELECT null, null, null INTO @topic_idplatform, @topic_idgroup, @topic_iddevice;

			SET @topic_aux = SUBSTRING( topic_var, LOCATE('/@P@', topic_var)+1 );
			SET @topic_idplatform = SUBSTRING( @topic_aux, (LOCATE('@P@', @topic_aux)+3), IF(LOCATE('/', @topic_aux), LOCATE('/', @topic_aux)-4, LENGTH(@topic_aux)) );
			SET @topic_aux = SUBSTRING( @topic_aux, LOCATE('/', @topic_aux)+1 );

			if LOCATE('@G@/', @topic_aux)
			then 
				SET @topic_aux = SUBSTRING( topic_var, LOCATE('/@G@', topic_var)+1 );
				SET @topic_group_path = SUBSTRING( @topic_aux, (LOCATE('@G@/', @topic_aux)+3), IF(LOCATE('/@', @topic_aux), LOCATE('/@', @topic_aux)-4, LENGTH(@topic_aux)) );
				SET @topic_idgroup = SUBSTRING(@topic_group_path, - LOCATE('/', REVERSE(@topic_group_path))+1 );
				SET @topic_aux = SUBSTRING( @topic_aux, LOCATE('/@', @topic_aux)+1 );
			end if;
			if LOCATE('@D@', @topic_aux)
			then 
				SET @topic_iddevice = SUBSTRING( @topic_aux, (LOCATE('@D@', @topic_aux)+3), IF(LOCATE('/', @topic_aux), LOCATE('/', @topic_aux)-4, LENGTH(@topic_aux)) );
			end if;
		END IF;
	END IF;

	-- SELECT @user, @user_aux, @idplatform, @idgroup, @iddevice, topic_var, @topic_idplatform, @topic_group_path, @topic_idgroup, @topic_iddevice;

	SET @forbidden = 0;
	if @topic_idplatform is not null and @topic_idplatform=@idplatform
	then
		DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups;
		CREATE TEMPORARY TABLE Temp_Table_groups (id int NOT NULL PRIMARY KEY AUTO_INCREMENT, idgroup int NOT NULL);

		if @idgroup is not null 
		then
			CALL e3tmqtt.Hierarchical_group_subgroups ( @idplatform, @idgroup );
		end if;

		if @topic_iddevice is not null and @topic_iddevice>0
		then
			if (select 1 from Temp_Table_groups where idgroup = (select idgroup from devices where idplatform=@idplatform and iddevice=@topic_iddevice limit 1)) is null
				AND @topic_iddevice != @iddevice
			then
				SET @forbidden=1;
			end if;
		end if;

		if @topic_idgroup is not null and @topic_idgroup != '0'
		then
			if @idgroup is not null
			then			
				SET @group_path = (	SELECT CONCAT( '/', GROUP_CONCAT(parent.idgroup SEPARATOR '/')) group_path
									FROM `groups` AS node,
										`groups` AS parent
									WHERE node.idplatform=@idplatform AND parent.idplatform=@idplatform 
										AND node.lft BETWEEN parent.lft AND parent.rgt
										AND node.idgroup = @idgroup
									ORDER BY node.lft );
				SET @topic_idgroup_path = IFNULL( (SELECT CONCAT( '/', GROUP_CONCAT(parent.idgroup SEPARATOR '/')) group_path
									FROM `groups` AS node,
										`groups` AS parent
									WHERE node.idplatform=@idplatform AND parent.idplatform=@idplatform 
										AND node.lft BETWEEN parent.lft AND parent.rgt
										AND node.idgroup = @topic_idgroup
									ORDER BY node.lft ), 0);
			end if;

			if @topic_group_path not like CONCAT(@group_path, '%') OR @topic_group_path != @topic_idgroup_path
			then
				SET @forbidden=1;
			end if;
		end if;
	end if;

	-- SELECT @user, @user_aux, @idplatform, @idgroup, @iddevice, topic_var, @topic_idplatform, @topic_group_path, @topic_idgroup, @topic_iddevice, @forbidden;

	SELECT 0, null, null, null, null INTO @result, @iduser, @idMQTT_acls, @topic, @rw;


	SELECT 1 result, ACLS.* FROM
	(
		select U.id iduser, A.id, A.topic, A.rw 
		from MQTT_acls A inner join MQTT_users U ON U.user=A.username
		where 	A.username=@user AND 
				A.rw like CONCAT('%', acc_var, '%') AND 
				topic_var REGEXP CONCAT('^', REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(A.topic, '#', '*'), '+', '[[:digit:]]+'), '%%idplatform%%', IFNULL(@topic_idplatform,0)), '%%idgroup%%', IFNULL(@topic_group_path,0)), '%%iddevice%%', IFNULL(@topic_iddevice,0)) )
				AND @forbidden=0
		union
		select D.id iduser, A.id, A.topic, A.rw 
		from MQTT_acls A inner join devices D ON D.mac=@mac_imei
		where 	A.username='DEVICE' AND A.rw like CONCAT('%', acc_var, '%') 
				AND topic_var REGEXP CONCAT('^', REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(A.topic, '#', '*'), '+', '[[:digit:]]+'), '%%idplatform%%', D.idplatform), '%%idgroup%%', D.idgroup), '%%iddevice%%', D.iddevice) )
	) ACLS limit 1
	INTO @result, @iduser, @idMQTT_acls, @topic, @rw;


	INSERT MQTT_AUTH_acl_log (date, iduser, username, clientid, topic, acc, mac_imei, result, idMQTT_acls, acl_topic, rw, request)
	SELECT NOW(), @iduser iduser, username_var, clientid_var, topic_var, acc_var, @user, @result, @idMQTT_acls, @topic, @rw, request_var;

	SELECT @result result, @iduser iduser;

	-- SELECT @user, @user_aux, @idplatform, @idgroup, @iddevice, acc_var, topic_var, @topic_idplatform, @topic_group_path, @topic_idgroup, @topic_iddevice, @forbidden;
end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.MQTT_HOOK_on_client_connected;
$$
create procedure e3tmqtt.MQTT_HOOK_on_client_connected ( action_var VARCHAR(200), client_id_var VARCHAR(200), username_var VARCHAR(200), conn_ack_var VARCHAR(45), request_var VARCHAR(2000) )
begin
	-- CALL e3tmqtt.MQTT_HOOK_on_client_connected ( 'client_connected', 'C_1492410235117', 'C_1492410235117', '0', '{"action":"client_connected","client_id":"C_1492410235117","username":"C_1492410235117","conn_ack":0}' );

	INSERT MQTT_HOOK_log (date, action, client_id, username, conn_ack, request)
	SELECT NOW(), action_var, client_id_var, username_var, conn_ack_var, request_var;

end $$ 
delimiter ;

delimiter $$
drop procedure if exists e3tmqtt.MQTT_HOOK_on_client_disconnected;
$$
create procedure e3tmqtt.MQTT_HOOK_on_client_disconnected ( action_var VARCHAR(200), client_id_var VARCHAR(200), username_var VARCHAR(200), reason_var VARCHAR(45), request_var VARCHAR(2000) )
begin
	-- CALL e3tmqtt.MQTT_HOOK_on_client_disconnected ( 'client_disconnected', 'C_1492410235117', 'C_1492410235117', 'normal', '{"action":"client_disconnected","client_id":"C_1492410235117","username":"C_1492410235117","reason":"normal"}' );

	INSERT MQTT_HOOK_log (date, action, client_id, username, reason, request)
	SELECT NOW(), action_var, client_id_var, username_var, reason_var, request_var;

end $$ 
delimiter ;

delimiter $$
drop procedure if exists e3tmqtt.MQTT_HOOK_on_client_subscribe;
$$
create procedure e3tmqtt.MQTT_HOOK_on_client_subscribe ( action_var VARCHAR(200), client_id_var VARCHAR(200), username_var VARCHAR(200), topic_var VARCHAR(45), opts_var VARCHAR(45), request_var VARCHAR(2000) )
begin
	-- CALL e3tmqtt.MQTT_HOOK_on_client_subscribe ( 'client_subscribe', 'C_1492410235117', 'C_1492410235117', 'world', '{"qos":0}', '{"action":"client_subscribe","client_id":"C_1492410235117","username":"C_1492410235117","topic":"world","opts":{"qos":0}}' );

	INSERT MQTT_HOOK_log (date, action, client_id, username, topic, opts, request)
	SELECT NOW(), action_var, client_id_var, username_var, topic_var, opts_var, request_var;

end $$ 
delimiter ;

delimiter $$
drop procedure if exists e3tmqtt.MQTT_HOOK_on_client_unsubscribe;
$$
create procedure e3tmqtt.MQTT_HOOK_on_client_unsubscribe ( action_var VARCHAR(200), client_id_var VARCHAR(200), username_var VARCHAR(200), topic_var VARCHAR(45), request_var VARCHAR(2000) )
begin
	-- CALL e3tmqtt.MQTT_HOOK_on_client_unsubscribe ( 'client_unsubscribe', 'C_1492410235117', 'C_1492410235117', 'world', '{"action":"client_unsubscribe","client_id":"C_1492410235117","username":"C_1492410235117","topic":"world"}' );

	INSERT MQTT_HOOK_log (date, action, client_id, username, topic, request)
	SELECT NOW(), action_var, client_id_var, username_var, topic_var, request_var;

end $$ 
delimiter ;

delimiter $$
drop procedure if exists e3tmqtt.MQTT_HOOK_on_session_created;
$$
create procedure e3tmqtt.MQTT_HOOK_on_session_created ( action_var VARCHAR(200), client_id_var VARCHAR(200), username_var VARCHAR(200), request_var VARCHAR(2000) )
begin
	-- CALL e3tmqtt.MQTT_HOOK_on_session_created ( 'session_created', 'C_1492410235117', 'C_1492410235117', '{"action":"session_created","client_id":"C_1492410235117","username":"C_1492410235117"}' );

	INSERT MQTT_HOOK_log (date, action, client_id, username, request)
	SELECT NOW(), action_var, client_id_var, username_var, request_var;

end $$ 
delimiter ;

delimiter $$
drop procedure if exists e3tmqtt.MQTT_HOOK_on_session_subscribed;
$$
create procedure e3tmqtt.MQTT_HOOK_on_session_subscribed ( action_var VARCHAR(200), client_id_var VARCHAR(200), username_var VARCHAR(200), topic_var VARCHAR(45), opts_var VARCHAR(45), request_var VARCHAR(2000) )
begin
	-- CALL e3tmqtt.MQTT_HOOK_on_session_subscribed ( 'session_subscribed', 'C_1492410235117', 'C_1492410235117', 'world', '{"qos":0}', '{"action":"session_subscribed","client_id":"C_1492410235117","username":"C_1492410235117","topic":"world","opts":{"qos":0}}' );

	INSERT MQTT_HOOK_log (date, action, client_id, username, topic, opts, request)
	SELECT NOW(), action_var, client_id_var, username_var, topic_var, opts_var, request_var;

end $$ 
delimiter ;

delimiter $$
drop procedure if exists e3tmqtt.MQTT_HOOK_on_session_unsubscribed;
$$
create procedure e3tmqtt.MQTT_HOOK_on_session_unsubscribed ( action_var VARCHAR(200), client_id_var VARCHAR(200), username_var VARCHAR(200), topic_var VARCHAR(45), request_var VARCHAR(2000) )
begin
	-- CALL e3tmqtt.MQTT_HOOK_on_session_unsubscribed ( 'session_unsubscribe', 'C_1492410235117', 'C_1492410235117', 'world', '{"action":"session_unsubscribe","client_id":"C_1492410235117","username":"C_1492410235117","topic":"world"}' );

	INSERT MQTT_HOOK_log (date, action, client_id, username, topic, request)
	SELECT NOW(), action_var, client_id_var, username_var, topic_var, request_var;

end $$ 
delimiter ;

delimiter $$
drop procedure if exists e3tmqtt.MQTT_HOOK_on_session_terminated;
$$
create procedure e3tmqtt.MQTT_HOOK_on_session_terminated ( action_var VARCHAR(200), client_id_var VARCHAR(200), username_var VARCHAR(200), reason_var VARCHAR(45), request_var VARCHAR(2000) )
begin
	-- CALL e3tmqtt.MQTT_HOOK_on_session_terminated ( 'session_terminated', 'C_1492410235117', 'C_1492410235117', 'normal', '{"action":"session_terminated","client_id":"C_1492410235117","username":"C_1492410235117","reason":"normal"}' );

	INSERT MQTT_HOOK_log (date, action, client_id, username, reason, request)
	SELECT NOW(), action_var, client_id_var, username_var, reason_var, request_var;

end $$ 
delimiter ;

delimiter $$
drop procedure if exists e3tmqtt.MQTT_HOOK_on_message_publish;
$$
create procedure e3tmqtt.MQTT_HOOK_on_message_publish ( action_var VARCHAR(200), from_client_id_var VARCHAR(200), from_username_var VARCHAR(200), topic_var VARCHAR(45), qos_var VARCHAR(45), retain_var VARCHAR(45), payload_var VARCHAR(2000), ts_var VARCHAR(45), request_var VARCHAR(2000) )
begin
	-- CALL e3tmqtt.MQTT_HOOK_on_message_publish ( 'message_publish', 'C_1492410235117', 'C_1492410235117', 'world', '0', 'true', 'Hello world', '1492412774', '{"action":"message_publish","from_client_id":"C_1492410235117","from_username":"C_1492410235117","topic":"world","qos":0,"retain":true,"payload":"Hello world!","ts":1492412774}' );

	INSERT MQTT_HOOK_log (date, action, from_client_id, from_username, topic, qos, retain, payload, ts, request)
	SELECT NOW(), action_var, from_client_id_var, from_username_var, topic_var, qos_var, retain_var, payload_var, ts_var, request_var;

end $$ 
delimiter ;

delimiter $$
drop procedure if exists e3tmqtt.MQTT_HOOK_on_message_delivered;
$$
create procedure e3tmqtt.MQTT_HOOK_on_message_delivered ( action_var VARCHAR(200), client_id_var VARCHAR(200), username_var VARCHAR(200), from_client_id_var VARCHAR(200), from_username_var VARCHAR(200), topic_var VARCHAR(45), qos_var VARCHAR(45), retain_var VARCHAR(45), payload_var VARCHAR(2000), ts_var VARCHAR(45), request_var VARCHAR(2000) )
begin
	-- CALL e3tmqtt.MQTT_HOOK_on_message_delivered ( 'message_delivered', 'C_1492410235117', 'C_1492410235117', 'world', '0', 'true', 'Hello world', '1492412774', '{"action":"message_delivered","from_client_id":"C_1492410235117","from_username":"C_1492410235117","topic":"world","qos":0,"retain":true,"payload":"Hello world!","ts":1492412774}' );

	INSERT MQTT_HOOK_log (date, action, client_id, username, from_client_id, from_username, topic, qos, retain, payload, ts, request)
	SELECT NOW(), action_var, client_id_var, username_var, from_client_id_var, from_username_var, topic_var, qos_var, retain_var, payload_var, ts_var, request_var;

end $$ 
delimiter ;

delimiter $$
drop procedure if exists e3tmqtt.MQTT_HOOK_on_message_acked;
$$
create procedure e3tmqtt.MQTT_HOOK_on_message_acked ( action_var VARCHAR(200), client_id_var VARCHAR(200), username_var VARCHAR(200), from_client_id_var VARCHAR(200), from_username_var VARCHAR(200), topic_var VARCHAR(45), qos_var VARCHAR(45), retain_var VARCHAR(45), payload_var VARCHAR(2000), ts_var VARCHAR(45), request_var VARCHAR(2000) )
begin
	-- CALL e3tmqtt.MQTT_HOOK_on_message_acked ( 'message_acked', 'C_1492410235117', 'C_1492410235117', 'world', '0', 'true', 'Hello world', '1492412774', '{"action":"message_acked","from_client_id":"C_1492410235117","from_username":"C_1492410235117","topic":"world","qos":0,"retain":true,"payload":"Hello world!","ts":1492412774}' );

	INSERT MQTT_HOOK_log (date, action, client_id, username, from_client_id, from_username, topic, qos, retain, payload, ts, request)
	SELECT NOW(), action_var, client_id_var, username_var, from_client_id_var, from_username_var, topic_var, qos_var, retain_var, payload_var, ts_var, request_var;

end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.LOGGER_message_log;
$$
create procedure e3tmqtt.LOGGER_message_log ( topic_var VARCHAR(200), payload_var VARCHAR(1000), mid_var VARCHAR(45), qos_var VARCHAR(45), retain_var VARCHAR(45), modelname_var VARCHAR(45), medium_var VARCHAR(45), devicename_var VARCHAR(45), version_var VARCHAR(45), serialNumber_var VARCHAR(45) , partner_var INT)
begin
	-- CALL e3tmqtt.LOGGER_message_log ( 'gateway/heartbeat', '{"modelname":"ES3020MA1E1D1-I1-PI1","devicename":"","power_source":"AC","version":"ADDM.3.2.2","mac_address":"34:FA:40:20:58:CA", "serial_number":"07970122050882","wwan_ip":"10.170.3.68","gWip":"10.64.18.236"}', 0, 0, '', 2 )

	IF(serialNumber_var IS NULL OR serialNumber_var='') AND (devicename_var IS NOT NULL AND LENGTH(devicename_var)>0)
    THEN
		SET serialNumber_var=(SELECT serialNumber FROM e3tmqtt.mqtt_logger_devices_meters WHERE devicename=devicename_var ORDER BY id DESC LIMIT 1);
	ELSE
		IF (SELECT 1 FROM e3tmqtt.mqtt_logger_devices_meters WHERE serialNumber=serialNumber_var AND devicename=devicename_var LIMIT 1) IS NULL
        THEN
			INSERT e3tmqtt.mqtt_logger_devices_meters (serialNumber, devicename, TS) SELECT serialNumber_var, devicename_var, now();
        END IF;
    END IF;

	INSERT e3tmqtt.mqtt_logger_log (TS, topic, payload, mid, qos, `retain`, modelname, `medium`, devicename, version, serialNumber, partner)
    SELECT now(), topic_var, payload_var, mid_var, qos_var, retain_var, modelname_var, medium_var, devicename_var, version_var, serialNumber_var, partner_var;
	    
end $$ 
delimiter ;


delimiter $$
drop procedure if exists e3tmqtt.MQTT_logger_log_day;
$$
create procedure e3tmqtt.MQTT_logger_log_day (  day_var DATE )
begin
	-- CALL e3tmqtt.MQTT_logger_log_day ( 0 );

	SET SQL_SAFE_UPDATES = 0;

	IF(day_var is null OR day_var=0 OR LENGTH(day_var)=0)
	THEN
		SET @today=date(date_add(now(), interval -1 day));
	ELSE
		SET @today=day_var;
	END IF;

	SET @next_day=date(date_add(@today, interval 1 day));

    
    DROP TEMPORARY TABLE IF EXISTS Temp_mqtt_meter_log;
    create temporary table Temp_mqtt_meter_log
	select date(TS) day, partner,count(distinct devicename) meters, count(*) messages from e3tmqtt.mqtt_logger_log where TS>=@today and TS<@next_day group by partner;

	if(select 1 from Temp_mqtt_meter_log limit 1) is not null
    then
		delete from e3tmqtt.mqtt_logger_log_day where day=@today;
		
		insert e3tmqtt.mqtt_logger_log_day (day, idpartner, gateways, gateways_messages)
		select date(TS) day, partner,count(distinct serialNumber) gateways, count(*) messages from e3tmqtt.mqtt_logger_log where TS>=@today and TS<@next_day group by partner;

		update e3tmqtt.mqtt_logger_log_day L inner join Temp_mqtt_meter_log T on T.day=L.day AND T.partner=L.idpartner
		set L.meters=T.meters, L.meters_messages=T.messages;
	end if;
end $$ 
delimiter ;




show procedure status;
