
DELIMITER $$
drop procedure if exists TRANGO_GET_DLMS_types;
$$
CREATE DEFINER=`root`@`%` PROCEDURE `TRANGO_GET_DLMS_types`()
BEGIN

SELECT * FROM e3t.middleware_DLMS_devices_types_BaseCamp;

END$$
DELIMITER ;

delimiter $$
drop procedure if exists TRANGO_GET_gateway_parameters_log;
$$
create procedure TRANGO_GET_gateway_parameters_log( token_var VARCHAR(45), iddevice_var int, date_start_var DATETIME, date_end_var DATETIME )
proc_label:begin

	CALL e3t.APP_get_token_validation (	@token_var:=token_var, 
									@iduser_var:=0, 
									@idgroup_var:=0, 
									@iddevice_var:=iddevice_var, 
									@idoutput_var:=0, 
									@result );
	if @result=0
	then
		SELECT 'Validation' ERROR;
		LEAVE proc_label;
	end if;	
	
	SET @consulta= CONCAT('
							SELECT date date_UTC, 
                            json_extract_1(Parameters,"APN") APN, 
							json_extract_1(Parameters,"RSSI") RSSI, 
							json_extract_1(Parameters,"RSRQ") RSRQ, 
							json_extract_1(Parameters,"RSRP") RSRP, 
							json_extract_1(Parameters,"SINR")/10 SINR, 
							json_extract_1(Parameters,"TX Power")/10 TX_Power, 
							json_extract_1(Parameters,"Cell Id") Cell_Id, 
							json_extract_1(Parameters,"Band") Band, 
                            json_extract_1(Parameters,"Firmware Version") Firmware_Version
							FROM middleware_udp_parameters_log  where iddevice=', iddevice_var);

    if(date_start_var!=0)
    then
        SET @consulta=CONCAT(@consulta, '
                            AND date >= ''', date_start_var,'''');
    end if;

    if(date_end_var!=0)
    then
        SET @consulta=CONCAT(@consulta, '
                            AND date <= ''', date_end_var,'''');
    end if;

    SET @consulta=CONCAT(@consulta, '
                            ORDER BY date DESC, id DESC');


    -- preparamos el objete Statement a partir de nuestra variable
    PREPARE smpt FROM @consulta;
    -- ejecutamos el Statement
    EXECUTE smpt;
    -- liberamos la memoria
    DEALLOCATE PREPARE smpt;
    
end $$



delimiter $$
drop procedure if exists TRANGO_GET_gateways;
$$
create procedure TRANGO_GET_gateways( token_var VARCHAR(45), idgroup_var INT, subgroups_var INT)
proc_label:begin

CALL e3t.APP_get_token_validation (	@token_var:=token_var, 
									@iduser_var:=0, 
									@idgroup_var:=idgroup_var, 
									@iddevice_var:=0, 
									@idoutput_var:=0, 
									@result );
	if @result=0
	then
		SELECT 'Validation' ERROR;
		LEAVE proc_label;
	end if;	

		SET SQL_SAFE_UPDATES = 0;

		DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups;
		CREATE TEMPORARY TABLE Temp_Table_groups (id int NOT NULL PRIMARY KEY AUTO_INCREMENT, idgroup int NOT NULL, KEY idgroup (idgroup));

		DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups_copy2;
		CREATE TEMPORARY TABLE Temp_Table_groups_copy2 (id int NOT NULL PRIMARY KEY AUTO_INCREMENT, idgroup int NOT NULL, KEY idgroup (idgroup));


	if (subgroups_var =0 OR subgroups_var IS NULL)
	then
		SET @row_number=0;
        
        CALL e3t.Hierarchical_group_subgroups ( idgroup_var );
        
        insert Temp_Table_groups_copy2 select * from Temp_Table_groups;
        
		SELECT 	
				D.id gateway_id, D.name gateway_name, D.idgroup group_id, G.name group_name, D.GPS_lat latitude, D.GPS_lng longitude
		from devices D
		inner join `groups` G on G.id=D.idgroup
		where D.idgroup=idgroup_var 
        group by D.id
		order by  D.name;
	else
		begin
            
			CALL e3t.Hierarchical_group_subgroups ( idgroup_var );
            
            insert Temp_Table_groups_copy2 select * from Temp_Table_groups;

			SET @row_number=0;
			SELECT 	
					D.id gateway_id, D.name gateway_name,  D.idgroup group_id, G.name group_name, D.GPS_lat latitude, D.GPS_lng longitude
			from devices D
			inner join `groups` G on G.id=D.idgroup
			INNER JOIN Temp_Table_groups TTG ON TTG.idgroup=D.idgroup
			group by D.id
			order by  D.name;
		end;
	end if;
end $$



delimiter $$
drop procedure if exists TRANGO_GET_groups;
$$
create procedure TRANGO_GET_groups( token_var VARCHAR(45), idgroup_var INT, subgroups_var INT)
proc_label:begin

    CALL e3t.APP_get_token_validation (    @token_var:=token_var,
                                    @iduser_var:=0,
                                    @idgroup_var:=idgroup_var,
                                    @iddevice_var:=0,
                                    @idoutput_var:=0,
                                    @result );
    if @result=0
    then
        SELECT 'Validation' ERROR;
        LEAVE proc_label;
    end if;   

    SET SQL_SAFE_UPDATES = 0;

    DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups;
    CREATE TEMPORARY TABLE Temp_Table_groups (id int NOT NULL PRIMARY KEY AUTO_INCREMENT, idgroup int NOT NULL, KEY idgroup (idgroup));

    DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups_copy;
    CREATE TEMPORARY TABLE Temp_Table_groups_copy (id int NOT NULL PRIMARY KEY AUTO_INCREMENT, idgroup int NOT NULL, KEY idgroup (idgroup));


    if (subgroups_var=0 OR subgroups_var IS NULL)
    then
        CALL e3t.Hierarchical_group_subgroups ( idgroup_var );

        insert Temp_Table_groups_copy select * from Temp_Table_groups;
        delete from Temp_Table_groups_copy where idgroup=idgroup_var;

        SELECT G.id group_id, G.name group_name, G.description group_description, G.parent group_parent_id,
                (select count(*) from `groups` inner join Temp_Table_groups_copy on Temp_Table_groups_copy.idgroup=groups.id where groups.lft>G.lft AND groups.lft<G.rgt) groups_inside,
                (select count(*) from devices inner join `groups` on groups.id=devices.idgroup inner join Temp_Table_groups on Temp_Table_groups.idgroup=groups.id where groups.lft BETWEEN G.lft AND G.rgt) gateways_inside
        FROM `groups` G
        WHERE parent=idgroup_var
        ORDER BY G.name;
    else
        begin
			CALL e3t.Hierarchical_group_subgroups ( idgroup_var );
          
            DROP TEMPORARY TABLE IF EXISTS Temp_Table_groups_list;
            CREATE TEMPORARY TABLE Temp_Table_groups_list (id int NOT NULL PRIMARY KEY AUTO_INCREMENT, idgroup int NOT NULL, KEY idgroup (idgroup));

            insert Temp_Table_groups_list select * from Temp_Table_groups;

            delete from Temp_Table_groups;
            CALL e3t.Hierarchical_group_subgroups ( idgroup_var );

            insert Temp_Table_groups_copy select * from Temp_Table_groups;

            SELECT     G.id group_id, G.name group_name, G.description group_description, G.parent group_parent_id, 
                    (select count(*) from `groups` inner join Temp_Table_groups_copy on Temp_Table_groups_copy.idgroup=groups.id where groups.lft>G.lft AND groups.lft<G.rgt) groups_inside,
                    (select count(*) from devices inner join `groups` on groups.id=devices.idgroup inner join Temp_Table_groups on Temp_Table_groups.idgroup=groups.id where groups.lft BETWEEN G.lft AND G.rgt) gateways_inside
            FROM `groups` AS G INNER JOIN
                    `groups` AS P
            LEFT JOIN Temp_Table_groups_list TTG ON TTG.idgroup=G.parent
            WHERE G.parent=TTG.idgroup
            AND G.lft BETWEEN P.lft AND P.rgt
            GROUP BY G.id
            ORDER BY G.name;

        end;
    end if;
end $$



delimiter $$
drop procedure if exists TRANGO_GET_water_quality_data;
$$
create PROCEDURE TRANGO_GET_water_quality_data( token_var VARCHAR(45), iddevice_var int, date_start_var DATETIME, date_end_var DATETIME)
proc_label:begin
	-- CALL e3t.TRANGO_GET_pressure_data ( 'b6158b9e0b8f5e560ad98e187fb17cb9', 508003047, 20, '2019-06-02', '2019-06-05' )


	CALL e3t.APP_get_token_validation (	@token_var:=token_var, 
									@iduser_var:=0, 
									@idgroup_var:=0, 
									@iddevice_var:=iddevice_var, 
									@idoutput_var:=0, 
									@result );
	if @result=0
	then
		SELECT 'Validation' ERROR;
		LEAVE proc_label;
	end if;	
	

	 SET @consulta= CONCAT('SELECT date date_UTC, round(json_extract_1(data,"5"),1) ph, round(json_extract_1(data,"6")) orp, round(json_extract_1(data,"1"),1) temperature
							FROM middleware_udp_frame_log  where iddevice=', iddevice_var);
  
    
    if(date_start_var!=0)
    then
        SET @consulta=CONCAT(@consulta, '
                            AND date >= ''', date_start_var,'''');
    end if;

    if(date_end_var!=0)
    then
        SET @consulta=CONCAT(@consulta, '
                            AND date <= ''', date_end_var,'''');
    end if;

    SET @consulta=CONCAT(@consulta, '
                            ORDER BY date DESC, id DESC');


    -- preparamos el objete Statement a partir de nuestra variable
    PREPARE smpt FROM @consulta;
    -- ejecutamos el Statement
    EXECUTE smpt;
    -- liberamos la memoria
    DEALLOCATE PREPARE smpt;
    
end $$

delimiter $$
drop procedure if exists TRANGO_GET_modbus_data;
$$
create PROCEDURE `TRANGO_GET_modbus_data`( token_var VARCHAR(45), iddevice_var int, date_start_var DATETIME, date_end_var DATETIME )
proc_label:begin
	-- CALL e3t.APP_get_MIDDLEWARE_UDP_sensor_synchronous_sender_log ( 'b6158b9e0b8f5e560ad98e187fb17cb9', 508003047, 20, '2019-06-02', '2019-06-05' )


	CALL e3t.APP_get_token_validation (	@token_var:=token_var, 
									@iduser_var:=0, 
									@idgroup_var:=0, 
									@iddevice_var:=iddevice_var, 
									@idoutput_var:=0, 
									@result );
	if @result=0
	then
		SELECT 'Validation' ERROR;
		LEAVE proc_label;
	end if;	
	
	SET @consulta= CONCAT('SELECT date date_UTC, data FROM middleware_udp_frame_log WHERE iddevice=', iddevice_var);
  
    if(date_start_var!=0)
    then
        SET @consulta=CONCAT(@consulta, '
                            AND date >= ''', date_start_var,'''');
    end if;

    if(date_end_var!=0)
    then
        SET @consulta=CONCAT(@consulta, '
                            AND date <= ''', date_end_var,'''');
    end if;

    SET @consulta=CONCAT(@consulta, '
                            ORDER BY date DESC, id DESC');


    -- preparamos el objete Statement a partir de nuestra variable
    PREPARE smpt FROM @consulta;
    -- ejecutamos el Statement
    EXECUTE smpt;
    -- liberamos la memoria
    DEALLOCATE PREPARE smpt;
    
end $$

delimiter $$
drop procedure if exists TRANGO_GET_modbus_data_raw;
$$
create PROCEDURE `TRANGO_GET_modbus_data_raw`( token_var VARCHAR(45), iddevice_var int, date_start_var DATETIME, date_end_var DATETIME )
proc_label:begin
	-- CALL e3t.APP_get_MIDDLEWARE_UDP_sensor_synchronous_sender_log ( 'b6158b9e0b8f5e560ad98e187fb17cb9', 508003047, 20, '2019-06-02', '2019-06-05' )


	CALL e3t.APP_get_token_validation (	@token_var:=token_var, 
									@iduser_var:=0, 
									@idgroup_var:=0, 
									@iddevice_var:=iddevice_var, 
									@idoutput_var:=0, 
									@result );
	if @result=0
	then
		SELECT 'Validation' ERROR;
		LEAVE proc_label;
	end if;	
	
	SET @consulta= CONCAT('SELECT date date_UTC, Frame data FROM middleware_udp_frame_log WHERE iddevice=', iddevice_var);
  
    if(date_start_var!=0)
    then
        SET @consulta=CONCAT(@consulta, '
                            AND date >= ''', date_start_var,'''');
    end if;

    if(date_end_var!=0)
    then
        SET @consulta=CONCAT(@consulta, '
                            AND date <= ''', date_end_var,'''');
    end if;

    SET @consulta=CONCAT(@consulta, '
                            ORDER BY date DESC, id DESC');


    -- preparamos el objete Statement a partir de nuestra variable
    PREPARE smpt FROM @consulta;
    -- ejecutamos el Statement
    EXECUTE smpt;
    -- liberamos la memoria
    DEALLOCATE PREPARE smpt;
    
end $$

delimiter $$
drop procedure if exists TRANGO_GET_sensor_data;
$$
create PROCEDURE `TRANGO_GET_sensor_data`( token_var VARCHAR(45), iddevice_var int, date_start_var DATETIME, date_end_var DATETIME, detail_var INT )
proc_label:begin
	-- CALL e3t.TRANGO_GET_pressure_data ( 'b6158b9e0b8f5e560ad98e187fb17cb9', 508003047, 20, '2019-06-02', '2019-06-05' )

	DECLARE pulse_var int;
	DECLARE sensor_var int;

	CALL e3t.APP_get_token_validation (	@token_var:=token_var, 
									@iduser_var:=0, 
									@idgroup_var:=0, 
									@iddevice_var:=iddevice_var, 
									@idoutput_var:=0, 
									@result );
	if @result=0
	then
		SELECT 'Validation' ERROR;
		LEAVE proc_label;
	end if;	
	

	SELECT ExtractValue(parameters, 'parameters/pulse_on'),  ExtractValue(descriptors, 'descriptors/sensor_1_id') INTO pulse_var, sensor_var from devices where id=iddevice_var;
	if (pulse_var=1)
	then
	   if(detail_var=1)
		then
		   SET @consulta= CONCAT('SELECT L.date date_UTC, json_extract_1(L.values_summary,"values_min") interval_minimum, json_extract_1(L.values_summary,"values_avg") interval_average, json_extract_1(L.values_summary,"values_max") interval_maximum,
								L.`values` interval_readings, T.name, T.description, T.unit FROM middleware_udp_sensor_synchronous_sender_log L 
								INNER JOIN middleware_sensors_type T ON T.id = ', sensor_var, 
                                ' WHERE L.idsensor_type=1 and L.iddevice=', iddevice_var);
	   
		else
		 SET @consulta= CONCAT('SELECT L.date date_UTC, json_extract_1(L.values_summary,"values_min") interval_minimum, json_extract_1(L.values_summary,"values_avg") interval_average, json_extract_1(L.values_summary,"values_max") interval_maximum,
								T.name, T.description, T.unit FROM middleware_udp_sensor_synchronous_sender_log L
                                INNER JOIN middleware_sensors_type T ON T.id = ', sensor_var, 
                                ' WHERE L.idsensor_type=1 and L.iddevice=', iddevice_var);
		end if;
    else
	
		 SET @consulta= CONCAT('SELECT L.date date_UTC, L.`values`, T.name, T.description, T.unit  FROM middleware_udp_sensor_log L 
          INNER JOIN middleware_sensors_type T ON T.id = ', sensor_var, 
         ' WHERE L.iddevice=', iddevice_var);
    end if;
    
    if(date_start_var!=0)
    then
        SET @consulta=CONCAT(@consulta, '
                            AND L.date >= ''', date_start_var,'''');
    end if;

    if(date_end_var!=0)
    then
        SET @consulta=CONCAT(@consulta, '
                            AND L.date <= ''', date_end_var,'''');
    end if;

    SET @consulta=CONCAT(@consulta, '
                            ORDER BY L.date DESC, L.id DESC');


    -- preparamos el objete Statement a partir de nuestra variable
	PREPARE smpt FROM @consulta;
    -- ejecutamos el Statement
	EXECUTE smpt;
    -- liberamos la memoria
	DEALLOCATE PREPARE smpt;
   -- select @consulta;
    
end $$

delimiter $$
drop procedure if exists TRANGO_GET_pressure_data;
$$
create PROCEDURE TRANGO_GET_pressure_data( token_var VARCHAR(45), iddevice_var int, date_start_var DATETIME, date_end_var DATETIME, detail_var INT )
proc_label:begin
	-- CALL e3t.TRANGO_GET_pressure_data ( 'b6158b9e0b8f5e560ad98e187fb17cb9', 508003047, 20, '2019-06-02', '2019-06-05' )


	CALL e3t.APP_get_token_validation (	@token_var:=token_var, 
									@iduser_var:=0, 
									@idgroup_var:=0, 
									@iddevice_var:=iddevice_var, 
									@idoutput_var:=0, 
									@result );
	if @result=0
	then
		SELECT 'Validation' ERROR;
		LEAVE proc_label;
	end if;	
	
   if(detail_var=1)
    then
       SET @consulta= CONCAT('SELECT date date_UTC, json_extract_1(values_summary,"values_min") interval_minimum, json_extract_1(values_summary,"values_avg") interval_average, json_extract_1(values_summary,"values_max") interval_maximum,
							`values` interval_readings FROM middleware_udp_sensor_synchronous_sender_log WHERE idsensor_type=1 and iddevice=', iddevice_var);
   
	else
	 SET @consulta= CONCAT('SELECT date date_UTC, json_extract_1(values_summary,"values_min") interval_minimum, json_extract_1(values_summary,"values_avg") interval_average, json_extract_1(values_summary,"values_max") interval_maximum
							FROM middleware_udp_sensor_synchronous_sender_log WHERE idsensor_type=1 and iddevice=', iddevice_var);
    end if;
    
    if(date_start_var!=0)
    then
        SET @consulta=CONCAT(@consulta, '
                            AND date >= ''', date_start_var,'''');
    end if;

    if(date_end_var!=0)
    then
        SET @consulta=CONCAT(@consulta, '
                            AND date <= ''', date_end_var,'''');
    end if;

    SET @consulta=CONCAT(@consulta, '
                            ORDER BY date DESC, id DESC');


    -- preparamos el objete Statement a partir de nuestra variable
    PREPARE smpt FROM @consulta;
    -- ejecutamos el Statement
    EXECUTE smpt;
    -- liberamos la memoria
    DEALLOCATE PREPARE smpt;
    
end $$

delimiter $$
drop procedure if exists TRANGO_GET_pulse_data;
$$
create PROCEDURE TRANGO_GET_pulse_data( token_var VARCHAR(45), iddevice_var int, date_start_var DATETIME, date_end_var DATETIME , detail_var INT )
proc_label:begin
	-- CALL e3t.APP_get_MIDDLEWARE_UDP_sensor_synchronous_sender_log ( 'b6158b9e0b8f5e560ad98e187fb17cb9', 508003047, 20, '2019-06-02', '2019-06-05' )


	CALL e3t.APP_get_token_validation (	@token_var:=token_var, 
									@iduser_var:=0, 
									@idgroup_var:=0, 
									@iddevice_var:=iddevice_var, 
									@idoutput_var:=0, 
									@result );
	if @result=0
	then
		SELECT 'Validation' ERROR;
		LEAVE proc_label;
	end if;	
	
	SELECT (ExtractValue(parameters, 'parameters/counter_factor')*ExtractValue(parameters, 'parameters/k1_factor')*ExtractValue(parameters, 'parameters/k2_factor'))/1000
	from devices where id=iddevice_var into @escale_factor;
   
   if(detail_var=1)
    then
	   SET @consulta= CONCAT('SELECT date date_UTC, round(json_extract_1(values_summary,"values_sum")) total_pulses, round(json_extract_1(values_summary,"values_sum")*@escale_factor,3) total_m3,
							`values` interval_pulses  FROM middleware_udp_sensor_synchronous_sender_log WHERE idsensor_type=6 and iddevice=', iddevice_var);
   
	else
		SET @consulta= CONCAT('SELECT date date_UTC, round(json_extract_1(values_summary,"values_sum")) total_pulses, round(json_extract_1(values_summary,"values_sum")*@escale_factor,3) total_m3
							FROM middleware_udp_sensor_synchronous_sender_log WHERE idsensor_type=6 and iddevice=', iddevice_var);
    end if;
    
    if(date_start_var!=0)
    then
        SET @consulta=CONCAT(@consulta, '
                            AND date >= ''', date_start_var,'''');
    end if;

    if(date_end_var!=0)
    then
        SET @consulta=CONCAT(@consulta, '
                            AND date <= ''', date_end_var,'''');
    end if;

    SET @consulta=CONCAT(@consulta, '
                            ORDER BY date DESC, id DESC');


    -- preparamos el objete Statement a partir de nuestra variable
    PREPARE smpt FROM @consulta;
    -- ejecutamos el Statement
    EXECUTE smpt;
    -- liberamos la memoria
    DEALLOCATE PREPARE smpt;
    
end $$


-- POST
delimiter $$
drop procedure if exists TRANGO_POST_login;
$$
create procedure TRANGO_POST_login( user_var VARCHAR(50), pass_var VARCHAR(50))
proc_label:begin

-- CALL e3t.Trango_post_login ('rober', '525118da19b674308b971936e29baa30')

	SELECT id, MD5(CONCAT(user, NOW(), pass))
	FROM users
	WHERE user = user_var and pass = pass_var and date_unregister is null
	LIMIT 1
	INTO @iduser, @token;

	if FOUND_ROWS()=1
	then
		INSERT users_tokens (iduser, date, token)
		SELECT @iduser, NOW(), @token;
	end if;

	SELECT 	
    U.user, 
	U.idgroup_home user_group_id,
	DATE_ADD(NOW(), INTERVAL U.token_expire_hours HOUR) token_expiration_date, 
    @token token
	FROM users U
	INNER JOIN platforms P ON P.id=U.idplatform
	WHERE U.user = user_var and U.pass = pass_var and U.date_unregister is null
	LIMIT 1;

end $$



